function [P_Psi,iflag] = Psi_fp_NAG(n,P_1,iflag)
% this program is used to solve the fixed point for P=Psi(theta,P)
%
%   INPUT
% P_1(:,j): conditional choice probabilities on d = 1 for firm j
% b: parameter vector
% f_s: transition matrix for exogenous state variable S
% SS: contains the value of S (ns x 1)
% AA: contains the value of (a_1,a_2,...,a_N) for each j\in A^{N} (na x
% nglob)
%
%   OUTPUT
% P_Psi = P - Psi(theta,P)
% I_dP = I - dPsi(theta,P)/dP 
%
% Written by Hiroyuki Kasahara, University of British Columbia 
% Last Updated: June 2011 

global theta f_s SS AA

b = theta;


%% parameters
eulerc = 0.5772;
bet = 0.96; % discount rate
P_1 = max(min(P_1,1-1e-8),1e-8);   
ns = 24;
nglob = 3;
P_1 = reshape(P_1,[ns,nglob]);
%[ns,nglob] = size(P_1); 
P_0 = 1 - P_1; 

na = 2^nglob;
nstate = ns/na;
npar_i = 4; 
%b_RS = b(1);  
%b_RN = b(2);
%b_EC = b(3);
b_FC = zeros(nglob,1);
b_FC(:,1) = b(4:3+nglob); 
theta_i = zeros(npar_i,nglob);
theta_i(1:4,:) = b(1:4,1)*ones(1,nglob);
theta_i(4,:) = b_FC';
  

%% (1) Compute Psi(theta,P) %%%%%%%%% 
P_jt = zeros(ns,na,nglob);
for j = 1:na
    for t = 1:nglob
        P_jt(:,j,t) = (P_1(:,t).*AA(j,t)+(1-P_1(:,t)).*(1-AA(j,t)));
    end
end
 
prod_P = ones(ns,na); %Pr(a|x) = \prod_{j=1}^N P_j(a_j|x)
for j = 1:na 
    for t = 1:nglob 
        prod_P(:,j) =  prod_P(:,j).*P_jt(:,j,t); 
    end 
end  
 
fs = kron(ones(na,na),f_s);
F_P = fs.*kron(prod_P,ones(1,nstate));
I_F = eye(ns) - bet*F_P;
%inv_I_F = inv(I_F); 

one_a = zeros(ns,nglob);
lnS = zeros(nstate,na);
lnS(:,:) = log(SS)*ones(1,na); 
lnS = lnS(:);    
for i = 1:nglob
    temp = zeros(nstate,na);
    temp(:,:) = ones(nstate,1)*(1-AA(:,i))'; 
    one_a(:,i) = temp(:);   
end
H_P = zeros(ns,nglob); 
for i = 1:nglob
    for j = 1:na  
        if (AA(j,i)==0) % \sum_{a_{-i}\in A^{N-1}}
            temp = ones(ns,1); %\prod_{j\neq i}P_j(a|x)
            for t = 1:nglob
                if (i~=t) 
                    temp =  temp.*P_jt(:,j,t);
                end %
            end
            H_P(:,i) = H_P(:,i) + temp.*log(1+sum(AA(j,:)')-AA(j,i));
        end
    end
end 

Z_P = zeros(ns,npar_i,nglob); 
z_P_0 = zeros(ns,npar_i,nglob); % = 0
z_P_1 = zeros(ns,npar_i,nglob);
lambda_P = zeros(ns,nglob);
for i = 1:nglob 
    z_P_1(:,:,i) = [lnS(:),-H_P(:,i),- one_a(:,i),-ones(ns,1)];
    Z_P(:,:,i)= repmat(P_1(:,i),1,npar_i).*squeeze(z_P_1(:,:,i)); %+ P_0.*z_P_0 (=0)
    lambda_P(:,i) = eulerc - (1-P_1(:,i)).*log((1-P_1(:,i))) - P_1(:,i).*log(P_1(:,i));
end 

Gamma = zeros(ns,nglob);
for i = 1:nglob
    Gamma(:,i) = I_F\(squeeze(squeeze(Z_P(:,:,i)))*theta_i(:,i)+lambda_P(:,i));
end 
 
Gamma_Z = zeros(ns,npar_i,nglob);
Gamma_lambda = zeros(ns,nglob);
for i = 1:nglob 
    Gamma_Z(:,:,i) =  I_F\squeeze(Z_P(:,:,i));
end
Gamma_lambda(:,:) = I_F\lambda_P; 

 
prod_P_0 = zeros(ns,na,nglob); 
prod_P_1 = zeros(ns,na,nglob);
for i = 1:nglob
    for j = 1:na
        prod_P_0(:,j,i) = ones(ns,1);
        prod_P_1(:,j,i) = ones(ns,1);
        for t = 1:nglob
            if (t~=i) 
                prod_P_0(:,j,i) =  prod_P_0(:,j,i).*P_jt(:,j,t);
                prod_P_1(:,j,i) =  prod_P_1(:,j,i).*P_jt(:,j,t);
            else
                prod_P_0(:,j,i) =  prod_P_0(:,j,i).*(1-AA(j,t)); % P_0(:,t) = 1
                prod_P_1(:,j,i) =  prod_P_1(:,j,i).*AA(j,t); % P_1(:,t) = 1
            end
        end
    end 
end  

D_F_P = zeros(ns,ns,nglob); % = f_i^P(x'|x,1) - f_i^P(x'|x,0)
for i = 1:nglob 
    D_F_P(:,:,i) = fs.*kron(squeeze(prod_P_1(:,:,i)-prod_P_0(:,:,i)),ones(1,nstate)); 
end  
 
D_z_P = z_P_1-z_P_0;
z_tilde = zeros(ns,npar_i,nglob);
lambda_tilde = zeros(ns,nglob);
for i = 1:nglob
    z_tilde(:,:,i) = D_z_P(:,:,i) + bet*squeeze(D_F_P(:,:,i))*Gamma_Z(:,:,i);
    lambda_tilde(:,i) = bet*squeeze(D_F_P(:,:,i))*Gamma_lambda(:,i);
end
 
Psi = zeros(ns,nglob);
for i = 1:nglob
    Psi(:,i) = 1./(1+exp(-squeeze(z_tilde(:,:,i))*theta_i(:,i)-lambda_tilde(:,i)));
end

P_Psi = P_1(:) - Psi(:);
 
%Psi = max(min(Psi,1-1e-8),1e-8);

% 
% %% (2) Compute dGamma/dP %%%%%%%%% 
% 
% %% (2-1) dF^P/dP %%%%
% % analytical derivatives
% dF_P = zeros(ns,ns,ns,nglob);
% for i_1 = 1:nglob
%     for i_2 = 1:ns   
%         dP_jt  = zeros(ns,na,nglob); 
%         for j = 1:na 
%            dP_jt(i_2,j,i_1) = AA(j,i_1)-(1-AA(j,i_1)); 
%         end  
%         dprod_P = ones(ns,na); %Pr(a|x) = \prod_{j=1}^N P_j(a_j|x)
%         for j = 1:na 
%             for t = 1:nglob 
%                 if t == i_1
%                     dprod_P(:,j) =  dprod_P(:,j).*dP_jt(:,j,t);
%                 else 
%                     dprod_P(:,j) =  dprod_P(:,j).*P_jt(:,j,t); 
%                 end 
%             end 
%         end 
%         dF_P(:,:,i_2,i_1) = fs.*kron(dprod_P,ones(1,nstate));
%     end
% end
% % using a numerical derivative (this will give the same value for dF_P)
% % h = 0.01*ones(ns,nglob);
% % th = P_1 + h;
% % h = th - P_1;   % avoid round-off error  
% % dF_P = zeros(ns,ns,ns,nglob);
% % for i_1 = 1:nglob
% %     for i_2 = 1:ns  
% %         P_2 = P_1;
% %         P_2(i_2,i_1) = P_1(i_2,i_1) + h(i_2,i_1);
% %         P_jt2 = zeros(ns,na,nglob);
% %         for j = 1:na
% %             for t = 1:nglob
% %                 P_jt2(:,j,t) = (P_2(:,t).*AA(j,t)+(1-P_2(:,t)).*(1-AA(j,t)));
% %             end
% %         end 
% %         prod_P2 = ones(ns,na); %Pr(a|x) = \prod_{j=1}^N P_j(a_j|x)
% %         for j = 1:na 
% %             for t = 1:nglob 
% %                 prod_P2(:,j) =  prod_P2(:,j).*P_jt2(:,j,t); 
% %             end 
% %         end
% %         dF_P(:,:,i_2,i_1) = (fs.*kron(prod_P2,ones(1,nstate))-F_P)./h(i_2,i_1);
% %     end
% % end
%            
%  
% %% (2-2) dGamma/dP=dGamma_P and dUpsilon/dP=dUpsilon_P %%%% 
% % we take analytical derivative for de^P/dP. On the other hand,
% % we use the existing program to take `numerical derivatives' for other functions
% % these numerical derivatives are accurate because they are linear in P
% h = ones(ns,nglob);
% th = P_1 + h;
% h = th - P_1;   % avoid round-off error  
% dGamma_P = zeros(ns,nglob,ns,nglob);
% dUpsilon_P = zeros(ns,nglob,ns,nglob);
% for i_1 = 1:nglob
%     for i_2 = 1:ns    
%         P_2 = P_1;
%         P_2(i_2,i_1) = P_1(i_2,i_1) + h(i_2,i_1);
%         P_jt_2 = zeros(ns,na,nglob);
%         for j = 1:na
%             for t = 1:nglob
%                 P_jt_2(:,j,t) = (P_2(:,t).*AA(j,t)+(1-P_2(:,t)).*(1-AA(j,t)));
%             end
%         end 
%         H_P_2 = zeros(ns,nglob); 
%         for i = 1:nglob
%             for j = 1:na  
%                 if (AA(j,i)==0) % \sum_{a_{-i}\in A^{N-1}}
%                     temp = ones(ns,1); %\prod_{j\neq i}P_j(a|x)
%                     for t = 1:nglob
%                         if (i~=t) 
%                             temp =  temp.*P_jt_2(:,j,t);
%                         end %
%                     end
%                     H_P_2(:,i) = H_P_2(:,i) + temp.*log(1+sum(AA(j,:)')-AA(j,i));
%                 end
%             end
%         end
%         Z_P_2 = zeros(ns,npar_i,nglob);
%        % z_P_0 = zeros(ns,npar_i,nglob); % = 0
%         z_P_1_2 = zeros(ns,npar_i,nglob); 
%         %lambda_P_2 = zeros(ns,nglob);
%         for i = 1:nglob 
%             z_P_1_2(:,:,i) = [lnS(:),-H_P_2(:,i),- one_a(:,i),-ones(ns,1)];
%             Z_P_2(:,:,i)= repmat(P_2(:,i),1,npar_i).*squeeze(z_P_1_2(:,:,i)); %+ P_0.*z_P_0 (=0)
%             %lambda_P_2(:,i) = eulerc - (1-P_2(:,i)).*log(1-P_2(:,i)) - P_2(:,i).*log(P_2(:,i)); 
%         end  
%         lambda_P_2 = lambda_P;
%         lambda_P_2(i_2,i_1) = lambda_P(i_2,i_1) + h(i_2,i_1)*log((1-P_1(i_2,i_1))/P_1(i_2,i_1)); % analytical derivative
%          
%         % dGamma_P 
%         for i = 1:nglob
%             dGamma_P(:,i,i_2,i_1) = I_F\(...
%                 (squeeze(Z_P_2(:,:,i)-Z_P(:,:,i))*theta_i(:,i)...
%                 +lambda_P_2(:,i)-lambda_P(:,i))./h(i_2,i_1)...
%                 +bet*squeeze(dF_P(:,:,i_2,i_1))*Gamma(:,i) );
%         end   
%         
%         % dUpsilon_P
%         prod_P_0_2 = zeros(ns,na,nglob); 
%         prod_P_1_2 = zeros(ns,na,nglob);
%         for i = 1:nglob
%             for j = 1:na
%                 prod_P_0_2(:,j,i) = ones(ns,1);
%                 prod_P_1_2(:,j,i) = ones(ns,1);
%                 for t = 1:nglob
%                     if (t~=i) 
%                         prod_P_0_2(:,j,i) =  prod_P_0_2(:,j,i).*P_jt_2(:,j,t);
%                         prod_P_1_2(:,j,i) =  prod_P_1_2(:,j,i).*P_jt_2(:,j,t);
%                     else
%                         prod_P_0_2(:,j,i) =  prod_P_0_2(:,j,i).*(1-AA(j,t)); % P_0(:,t) = 1
%                         prod_P_1_2(:,j,i) =  prod_P_1_2(:,j,i).*AA(j,t); % P_1(:,t) = 1
%                     end
%                 end
%             end 
%         end   
%         D_F_P_2 = zeros(ns,ns,nglob); % = f_i^P(x'|x,1) - f_i^P(x'|x,0)
%         for i = 1:nglob 
%             D_F_P_2(:,:,i) = fs.*kron(squeeze(prod_P_1_2(:,:,i)-prod_P_0_2(:,:,i)),ones(1,nstate)); 
%         end    
%         dz_tilde_2 = zeros(ns,npar_i,nglob);
%         dlambda_tilde_2 = zeros(ns,nglob);
%         for i = 1:nglob
%             dz_tilde_2(:,:,i) = (z_P_1_2(:,:,i)-z_P_1(:,:,i))./h(i_2,i_1)...
%                 + bet*(squeeze(D_F_P_2(:,:,i)-D_F_P(:,:,i))./h(i_2,i_1))*Gamma_Z(:,:,i);
%             dlambda_tilde_2(:,i) = bet*(squeeze(D_F_P_2(:,:,i)-D_F_P(:,:,i))./h(i_2,i_1))*Gamma_lambda(:,i);
%         end
%         for i = 1:nglob
%             dUpsilon_P(:,i,i_2,i_1)= Psi(:,i).*(1-Psi(:,i)).*( ... 
%                 squeeze(dz_tilde_2(:,:,i))*theta_i(:,i) + dlambda_tilde_2(:,i) );
%         end 
%     end
% end 
% % using a numerical derivative 
% % h = 0.00001*ones(ns,nglob);
% % th = P_1 + h;
% % h = th - P_1;   % avoid round-off error  
% % dGamma_P = zeros(ns,nglob,ns,nglob);
% % for i_1 = 1:nglob
% %     for i_2 = 1:ns     
% %         P_2 = P_1;
% %         P_2(i_2,i_1) = P_1(i_2,i_1) + h(i_2,i_1);
% %         P_jt_2 = zeros(ns,na,nglob); 
% %         for j = 1:na
% %             for t = 1:nglob
% %                 P_jt_2(:,j,t) = (P_2(:,t).*AA(j,t)+(1-P_2(:,t)).*(1-AA(j,t)));
% %             end
% %         end 
% %         prod_P_2 = ones(ns,na); %Pr(a|x) = \prod_{j=1}^N P_j(a_j|x)
% %         for j = 1:na 
% %             for t = 1:nglob 
% %                 prod_P_2(:,j) =  prod_P_2(:,j).*P_jt_2(:,j,t); 
% %             end 
% %         end   
% %         fs = kron(ones(na,na),f_s);
% %         F_P_2 = fs.*kron(prod_P_2,ones(1,nstate));
% %         I_F_2 = eye(ns) - bet*F_P_2;
% %         %inv_I_F = inv(I_F); 
% %  
% %         H_P_2 = zeros(ns,nglob); 
% %         for i = 1:nglob
% %             for j = 1:na  
% %                 if (AA(j,i)==0) % \sum_{a_{-i}\in A^{N-1}}
% %                     temp = ones(ns,1); %\prod_{j\neq i}P_j(a|x)
% %                     for t = 1:nglob
% %                         if (i~=t) 
% %                             temp =  temp.*P_jt_2(:,j,t);
% %                         end %
% %                     end
% %                     H_P_2(:,i) = H_P_2(:,i) + temp.*log(1+sum(AA(j,:)')-AA(j,i));
% %                 end
% %             end
% %         end  
% %         Z_P_2 = zeros(ns,npar_i,nglob);  
% %         z_P_1_2 = zeros(ns,npar_i,nglob);
% %         lambda_P_2 = zeros(ns,nglob);
% %         for i = 1:nglob 
% %             z_P_1_2(:,:,i) = [lnS(:),-H_P_2(:,i),- one_a(:,i),-ones(ns,1)];
% %             Z_P_2(:,:,i)= repmat(P_2(:,i),1,npar_i).*squeeze(z_P_1_2(:,:,i)); %+ P_0.*z_P_0 (=0)
% %             lambda_P_2(:,i) = eulerc - (1-P_2(:,i)).*log(1-P_2(:,i)) - P_2(:,i).*log(P_2(:,i));
% %         end  
% %         Gamma_2 = zeros(ns,nglob);
% %         for i = 1:nglob
% %             Gamma_2(:,i) = I_F_2\(squeeze(Z_P_2(:,:,i))*theta_i(:,i)+lambda_P_2(:,i));
% %         end 
% %         dGamma_P(:,:,i_2,i_1)= (Gamma_2-Gamma)./h(i_2,i_1);
% %     end
% % end
% 
% 
%  
% %% (2-3) dUpsilon/dV=dUpsilon_V %%%% 
%  
% dUpsilon_V = zeros(ns,ns,nglob); 
% for i = 1:nglob
%     dUpsilon_V(:,:,i) = ((Psi(:,i).*(1-Psi(:,i)))*ones(1,ns)).*(bet*D_F_P(:,:,i));
% end
%    
% %% (2-4) dPsi/dP using chain rules and analytical derivatives
% dPsi_P = zeros(ns,nglob,ns,nglob); 
% for i = 1:nglob
%     for j = 1:nglob
%         if i==j
%             dPsi_P(:,i,:,j) = squeeze(dUpsilon_V(:,:,i))*squeeze(dGamma_P(:,i,:,j));
%         else %if i~=j
%             dPsi_P(:,i,:,j) = squeeze(dUpsilon_V(:,:,i))*squeeze(dGamma_P(:,i,:,j))...
%                 + squeeze(dUpsilon_P(:,i,:,j));
%         end
%     end
% end
%           
% % reshape dPsi_P
% dP = zeros(ns*nglob,ns*nglob);
% for i = 1:nglob
%     for j = 1:ns
%         dP(:,j+(i-1)*ns) = reshape(dPsi_P(:,:,j,i),[ns*nglob,1]);
%     end
% end
% 
% I_dP = eye(ns*nglob,ns*nglob) - dP;
 
return








