function Sigma = Sigma_PS(P_1,P_x,AA)
% Computing the asymptotic variance-covariance matrix for a frequency estimator of n^{-1/2}P(a|x)
% This is used for computing the optimal weighting matrix for Asymptotic
% Least Squares estimator by Pesendorfer and Schmidt-Denger.
%
% The variance formula is derived by delta method 
%
%   INPUT
% P_1(:,j): a frequency estimator of conditional choice probabilities: d = 1 for firm j
% P_x: a frequency estimator for x_0 
% AA: contains the value of (a_1,a_2,...,a_N) for each j\in A^{N} (na x
% nglob) 
%
%   OUTPUT
% Sigma = asymptotic variance for a frequency estimator of P(a|x)
%
% Written by Hiroyuki Kasahara, UBC
% Last Updated: July 2009

[ns,nglob] = size(P_1);
ns = size(P_x,1);
[na,nglob] = size(AA);
nstate = ns/na;
one_AA = 1-AA;

P_0 = 1-P_1; 
P_a = zeros(na,ns);
P_z = zeros(na,ns); % joint empirical distribution of z=(a_1,x_0)
for i = 1:na 
    P_a(i,:) = prod(P_1.*repmat(AA(i,:),ns,1)+P_0.*repmat(one_AA(i,:),ns,1),2)';
    P_z(i,:) = P_a(i,:).*P_x';
    % [a_1,a_2,a_3] = AA(i,:);
end
P_z = P_z(:); % na*ns = 192   

C_p = zeros(ns*nglob,na*ns);
for i = 1:ns
    c = zeros(ns*na,1);
    c(1+na*(i-1):na*i,1) = ones(na,1);
    % P_x(i) = c'*P_z
    for j = 1:nglob
        b = zeros(ns*na,1);
        b(1+na*(i-1):na*i,1) = AA(:,j);
        % P_1(i,j) = (b'*P_z)/(c'*P_z) 
        C_p(i+ns*(j-1),:) = (b'*P_x(i)-c'*(b'*P_z))/(P_x(i)^2); % delta method
    end
end
 
Omega = zeros(na*ns,na*ns); % covariance matrix for multinomial distribution of z=(a_1,x_0)
for i = 1:na*ns
    for j = 1:i
        if (j == i)
            Omega(i,j) = P_z(i)*(1-P_z(i));
        else % if j<i
            Omega(i,j) = -P_z(i)*P_z(j);
            Omega(j,i) = Omega(i,j);
        end
    end
end

Sigma = C_p*Omega*C_p';


            
            
            
