function Z = basis_RPM(dPsi_dp,delta_RPM)  
% basis_RPM computes an orthonomal basis for the invariant subspace of dPsi_dP  
% associated with the eigenvalues of which modulus are larger than delta_RPM
%
% See section 4.1 of Sheroff and Keller (1993) for details
%
%   INPUT
% dPsi_dP: Jacobian of Psi mapping with respect to P
% delta_RPM: criteria for splitting the space of dPsi_dp
%
%   OUTPUT
% Z: an orthonormal basis
% 
% Written by Hiroyuki Kasahara, University of British Columbia 
% Last Updated: July 2009 

nconv = 500;
n = size(dPsi_dp,1);

% initialization
dP = dPsi_dp;
Z = [];   
t = 0;

% t = 1 --- we make sure that Z is not empty
dq = ones(n,1)+1i;  % the eigenvectors are in complex space
ii = 1;
while (ii<nconv)
    dq0 = dq;
    dq = dP*dq; 
    dq = (1/sqrt(sum(dq.^2)))*dq; 
    if (sqrt(sum((dq0-dq).^2))<1e-4)|(sqrt(sum((dq0+dq).^2))<1e-4) % the latter is for the case of lambda<0
        lambda_max = dq'*(dP*dq); 
        dz = dq; 
        t = t + 1;  
        break;
    end 
    ii = ii + 1;
end  
if (ii==nconv)
    D = [dq dq0];
    [D,R] = qr(D); % QR decomposition
    if (abs(R(1,1))>1000*abs(R(2,2))) % then add one new vector
        dq = D(:,1); 
        lambda_max = dq'*dP*dq;
        dz = dq; 
        t = t + 1;
    else % then (dq,dq0) is complex conjugate, add two new vectors  
        dz = D(:,1:2); 
        lambda_max = D(:,1)'*dP*D(:,1);
        t = t + 2; 
    end
end 
Z = [Z dz];
Q = eye(n) - Z*Z';
dP = Q*dPsi_dp*Q; % update dP 

% t>1
while (t<n)
    dq = ones(n,1)+1i; % the eigenvectors are complex space
    ii = 1;
    while (ii<nconv)
        dq0 = dq;
        dq = dP*dq;
        dq = (1/sqrt(sum(dq.^2)))*dq; 
        if (sqrt(sum((dq0-dq).^2))<1e-4)|(sqrt(sum((dq0+dq).^2))<1e-4) % the latter is for the case of lambda<0
            lambda_max = dq'*dP*dq;
            dz = dq; 
            t = t + 1;  
            break;
        end 
        ii = ii + 1;
    end 
    if (ii==nconv)
        D = [dq dq0];
        [D,R] = qr(D); % QR decomposition
        if (abs(R(1,1))>1000*abs(R(2,2))) % then add one new vector
            dq = D(:,1); 
            lambda_max = dq'*dP*dq;
            dz = dq; 
            t = t + 1;
        else % then (dq,dq0) is complex conjugate, add two new vectors  
            dz = D(:,1:2); 
            lambda_max = D(:,1)'*dP*D(:,1);
            t = t + 2; 
            %disp('adding two new vectors in basis_RPM');
        end
    end 
    if abs(lambda_max)<delta_RPM, break, end 
    Z = [Z dz];
    Q = eye(n) - Z*Z';
    dP = Q*dPsi_dp*Q; % update dP 
end 
            
