function dP = dPsi_theta(P_1,b,f_s,SS,AA)
% compute the derivative of Psi with respect to theta
%
%   INPUT
% P_1(:,j): conditional choice probabilities on d = 1 for firm j
% b: parameter vector
% f_s: transition matrix for exogenous state variable S
% AA: contains the value of (a_1,a_2,...,a_N) for each j\in A^{N} (na x
% nglob)
% SS: contains the value of S (ns x 1)
%
%   OUTPUT
% dP(:,j): derivative of Psi with respect to b(j)
%
% Written by Hiroyuki Kasahara, UBC
% Last Updated: July 2009


% parameters
eulerc = 0.5772;
bet = 0.96; % discount rate
P_1 = max(min(P_1,1-1e-8),1e-8);   
[ns,nglob] = size(P_1); 
P_0 = 1 - P_1; 

na = 2^nglob;
nstate = ns/na;
npar_i = 4; 
b_RS = b(1);  
b_RN = b(2);
b_EC = b(3);
b_FC = zeros(nglob,1);
b_FC(:,1) = b(4:3+nglob); 
theta_i = zeros(npar_i,nglob);
theta_i(1:4,:) = b(1:4,1)*ones(1,nglob);
theta_i(4,:) = b_FC';
  

%%%%%%% Computing in a strightforward way %%%%%%%%%


P_jt = zeros(ns,na,nglob);
for j = 1:na
    for t = 1:nglob
        P_jt(:,j,t) = (P_1(:,t).*AA(j,t)+P_0(:,t).*(1-AA(j,t)));
    end
end

% (1) Transition function
prod_P = ones(ns,na); %Pr(a|x) = \prod_{j=1}^N P_j(a_j|x)
for j = 1:na 
    for t = 1:nglob 
        prod_P(:,j) =  prod_P(:,j).*P_jt(:,j,t); 
    end 
end  


fs = kron(ones(na,na),f_s);
F_P = fs.*kron(prod_P,ones(1,nstate));
I_F = eye(ns) - bet*F_P;
%inv_I_F = inv(I_F); 

one_a = zeros(ns,nglob);
lnS = zeros(nstate,na);
lnS(:,:) = log(SS)*ones(1,na); 
lnS = lnS(:);    
for i = 1:nglob
    temp = zeros(nstate,na);
    temp(:,:) = ones(nstate,1)*(1-AA(:,i))'; 
    one_a(:,i) = temp(:);   
end
H_P = zeros(ns,nglob); 
for i = 1:nglob
    for j = 1:na  
        if (AA(j,i)==0) % \sum_{a_{-i}\in A^{N-1}}
            temp = ones(ns,1); %\prod_{j\neq i}P_j(a|x)
            for t = 1:nglob
                if (i~=t) 
                    temp =  temp.*P_jt(:,j,t);
                end %
            end
            H_P(:,i) = H_P(:,i) + temp.*log(1+sum(AA(j,:)')-AA(j,i));
        end
    end
end 

Z_P = zeros(ns,npar_i,nglob); 
z_P_0 = zeros(ns,npar_i,nglob); % = 0
z_P_1 = zeros(ns,npar_i,nglob);
lambda_P = zeros(ns,nglob);
for i = 1:nglob 
    z_P_1(:,:,i) = [lnS(:),-H_P(:,i),- one_a(:,i),-ones(ns,1)];
    Z_P(:,:,i)= repmat(P_1(:,i),1,npar_i).*squeeze(z_P_1(:,:,i)); %+ P_0.*z_P_0 (=0)
    lambda_P(:,i) = eulerc - P_0(:,i).*log(P_0(:,i)) - P_1(:,i).*log(P_1(:,i));
end
 
Gamma_Z = zeros(ns,npar_i,nglob);
Gamma_lambda = zeros(ns,nglob);
for i = 1:nglob 
    Gamma_Z(:,:,i) = I_F\squeeze(Z_P(:,:,i));
end
Gamma_lambda(:,:) = I_F\lambda_P; 

 
prod_P_0 = zeros(ns,na,nglob); 
prod_P_1 = zeros(ns,na,nglob);
for i = 1:nglob
    for j = 1:na
        prod_P_0(:,j,i) = ones(ns,1);
        prod_P_1(:,j,i) = ones(ns,1);
        for t = 1:nglob
            if (t~=i) 
                prod_P_0(:,j,i) =  prod_P_0(:,j,i).*P_jt(:,j,t);
                prod_P_1(:,j,i) =  prod_P_1(:,j,i).*P_jt(:,j,t);
            else
                prod_P_0(:,j,i) =  prod_P_0(:,j,i).*(1-AA(j,t)); % P_0(:,t) = 1
                prod_P_1(:,j,i) =  prod_P_1(:,j,i).*AA(j,t); % P_1(:,t) = 1
            end
        end
    end 
end  

D_F_P = zeros(ns,ns,nglob); % = f_i^P(x'|x,1) - f_i^P(x'|x,0)
for i = 1:nglob 
    D_F_P(:,:,i) = fs.*kron(squeeze(prod_P_1(:,:,i)-prod_P_0(:,:,i)),ones(1,nstate)); 
end  


D_z_P = z_P_1-z_P_0;
z_tilde = zeros(ns,npar_i,nglob);
lambda_tilde = zeros(ns,nglob);
for i = 1:nglob
    z_tilde(:,:,i) = D_z_P(:,:,i) + bet*squeeze(D_F_P(:,:,i))*Gamma_Z(:,:,i);
    lambda_tilde(:,i) = bet*squeeze(D_F_P(:,:,i))*Gamma_lambda(:,i);
end
 
P = zeros(ns,nglob);
for i = 1:nglob
    P(:,i) = 1./(1+exp(-squeeze(z_tilde(:,:,i))*theta_i(:,i)-lambda_tilde(:,i)));
end 

P = max(min(P,1-1e-8),1e-8);  
  
% compute the derivative of Psi with respect to theta
dP_i = zeros(ns,nglob,2);
for j = 1:2
    for i = 1:nglob
        P_i = P(:,i);
        %dP = P.*(1-P).*squeeze(zz(:,j,i));
        dP_1 = P_i.*(1-P_i).*squeeze(z_tilde(:,j,i)); 
        dP_i(:,i,j) = dP_1; % ns*nglob by 1
    end
end
dP = zeros(ns*nglob,2);
for j = 1:2
    temp = dP_i(:,:,j);
    dP(:,j) = temp(:);
end
 

return








