% mpe_PS.m
%
% This program conducts Monte Carlo simulation for the paper
% ``Sequential Estimation of Structural Models with Fixed Point
% Constraint''
%
% Reference:
% Aguirregabiria and Mira (2007) Econometrica paper
% Pesendorfer and Schmidt-Dengler (2008) REStud paper
%
% Written by Hiroyuki Kasahara, University of British Columbia 
% Last Updated: July 2009 

% mpe_PS.m: Simulation using Asymptotic LS estimator 


clear all
 
c1 = clock;     % record the time the program started
randn('state',[89234242; 32475543]); % seed for random normal variates
rand('state',89234241); % seed for random uniform variates


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%% SPECIFICATION %%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
% constant
eulerc = 0.5772;
convcrit = 1000;
convcons = 1e-5;
max_iter = 50;

% Set the values for Monte Carlo experiments
nobs = 8000; % the number of markets
nsim = 1000; % the number of simulations
q = 4; % the number of Psi iterations in defining Psi_tilde_q
np = 2; % the number of parameters estimated

% Model's parameter
nglob = 3; % the number of global firms
nstate = 3; % the number of market sizes
npar1 = 3; % the number of parameters that are common across firms
npar2 = nglob*1; % the number of parameters that are different across firms
npar = npar1 + npar2;
npar_i = 4;
na = 2^nglob;
ns = nstate*na; % = |X|
sigma = 1;
% parameter theta
theta = zeros(npar,1); %specify the parameter values:
theta_RS = 1;
theta_RN = 2; % strategic substitutability
theta_EC = 1; 
theta_FC(:,1) = [1.0;0.9;0.8];   
theta(:,1) = [theta_RS;theta_RN;theta_EC;theta_FC];
% transition function for S_{mt}
SS = zeros(nstate,1);
f_s = zeros(nstate,nstate);  % transition function for S_mt 
SS(:,1) = 2*[1;3;5];
f_s(:,:) = [0.8,0.2,0.0;0.2,0.6,0.2;0.0,0.2,0.8];  
% AA(x_t,i) is the value of a_{i,t-1} given x_t = (a_{1,t-1},a_{2,t-1},a_{3,t-1})
AA = zeros(na,nglob); 
for i_1 = 0:1
    for i_2 = 0:1
        for i_3 = 0:1
            i = i_1*4 + i_2*2 + i_3+1;
            AA(i,:) = [i_1,i_2,i_3];
        end
    end
end 


%%%%%%  Choose an initial estimator for P %%%%%%

spec_P0 = 'FQ'; % {'LG','FQ','TR','RD','UN'}
% LG = Logit
% FQ = Frequency
% TR = True    (=P_1_star)
% RD = Random  (drawn from U(0,1))
% UN = Uniform (i.e., set to 0.5)

% initial guess of alpha
alpha_0 = 0.2; %  \tilde Psi = [Psi].^alpha .* P.^(1-alpha)
delta_RPM = 0.5; % to define an orthogonal subspace for RPM
delta_RPM2 = 0.8; % to define an orthogonal subspace for RPM
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% Solving the fixed point %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (1) (a) compute the fixed point under \Psi, 
%     (b) compute the fixed point under \Lambda = \Psi^{alpha P^{1-alpha}
%     (c) compute the fixed point under \Gamma = RPM mapping and the Least
%         Squares estimator from random starting values
%     (d) compute the optimal alpha
% (2) obtain the steady state distribution p^*(S_t,a_{t-1})
% (3) obtain cdf for p^*(S_t,a_{t-1})
% (4) Infeasible optimal weighting matrix for the ASL estimator 
% (5) Descriptive statistics for the model

% (1)-(a) Compute the fixed point under \Psi
P_1_star = 0.5*ones(ns,nglob);
Psi_conviter = zeros(2,1);
for j = 1:1000
    P = P_1_star;
    P_1_star = Psi(P,theta,f_s,SS,AA);
    if sum(abs(P(:)-P_1_star(:)))<convcons
        Psi_conviter(1) = j;
        break
    end
end
dummy_period2 = 0;
if j>=1000
    disp('P does not converge under Psi after 1000 iterations.');
    P_2 = Psi(P_1_star,theta,f_s,SS,AA); 
    P_3 = Psi(P_2,theta,f_s,SS,AA); 
    disp('The followings are three sequences, P_j, P_{j+1}, P_{j+2}');
    disp('P_j=');disp(P);
    disp('P_{j+1}=');disp(P_1_star);
    disp('P_{j+2}=');disp(P_2);
    disp('P_{j+3}=');disp(P_3);
    P_psi_0 = P;
    P_psi_1 = P_1_star;
    P_psi_2 = P_2;
    if abs(P(:)-P_2(:))<convcons
        disp('The sequence \{P_j\} converges to period-2 cycle');
        disp(' ');
        dummy_period2 = 1;
    end
end

% (1)-(b) Compute the fixed point under Lambda mapping
P_1_star = 0.5*ones(ns,nglob);
P = P_1_star;
for j = 1:5000
    P = P_1_star;
    P_1_star = Lambda(P_1_star,theta,f_s,SS,AA,alpha_0);
    if sum(abs(P(:)-P_1_star(:)))<convcons, break, end
end
if j >= 5000
    disp('P does not converge under Lambda= after 1000 iterations. Stopping...');
    stop
end
P_0_star = 1 - P_1_star;
  
% (1)-(c) Compute the fixed point under \Lambda_RPM using different initial
% conditions --- check multiple fixed points
options = optimset('Display','off','GradObj','off','LargeScale','off','MaxFunEvals',10000,'TolFun',1e-6,'TolX',1e-6);
P_star = zeros(ns,nglob,100);
for i = 1:10
    P_1 = 0.05*ones(ns,nglob)+0.9*rand(ns,nglob);
    P_1 = P;
    for j = 1:50
        P = P_1; 
        if isinteger(j/5)|(j==1)
            dPsi_dp = Psi_P('Psi',P_1,theta,f_s,SS,AA);  
            dPsi_dp = real(dPsi_dp);
            Z = basis_RPM(dPsi_dp,delta_RPM); 
            inv_I_f = inv(eye(size(Z,2))-Z'*dPsi_dp*Z); 
        end
        P_1 = RPM(P_1,theta,f_s,SS,AA,inv_I_f,Z); 
        if sum(abs(P(:)-P_1(:)))<convcons, break, end
    end
    W = eye(ns*nglob);
    P_1 = fminunc('LS',P,options,theta,f_s,SS,AA,W);   
    P_2 = Psi(P_1,theta,f_s,SS,AA);
    if sum(abs(P_1(:)-P_2(:)))<convcons
        if sum(max(P_1(:) - P_1_star(:)))>convcons*10 
            disp('Multiple fixed points found in Lambda_RPM mapping. Stopping...');
            disp('P under Lambda =');disp(P_1_star);
            disp('P under Lambda_RPM =');disp(P_1); 
            stop 
        end
    else
        disp('Lambda_RPM mapping did not converge'); 
        disp('P_j=');disp(P_1);
        disp('P_{j+1}=');disp(P_2);
    end
end  


% (1)-(d) compute the ``optimal'' alpha to define Lambda mapping
dP = Psi_P('Psi',P_1_star,theta,f_s,SS,AA);
eig_dp = eig(dP); 
b = fminsearch('eig_Lambda',1,options,eig_dp);  
alpha = 1/(1+abs(b)); 
disp('alpha =');disp(alpha); 

% (2) Obtain the steady state distribution p^*(S_t,a_{t-1})
prod_P_star = zeros(ns,na); %Pr(a|x) = \prod_{j=1}^N P_j(a_j|x)
for j = 1:na
    prod_P_star(:,j) = ones(ns,1);
    for t = 1:nglob
        prod_P_star(:,j) =  prod_P_star(:,j).*(P_1_star(:,t).*AA(j,t)+P_0_star(:,t).*(1-AA(j,t)));
    end
end
F_P_star = kron(ones(na,na),f_s).*kron(prod_P_star,ones(1,nstate));
p_star = (1/ns)*ones(ns,1);
for j = 1:1000
    p_star_0 = p_star;
    p_star = F_P_star'*p_star_0;
    if sum(abs(p_star-p_star_0))<convcons, break, end
end
p_star = p_star./sum(p_star(:)); % to avoid numerical errors

% (3) Obtain cdf for p^*(S_t,a_{t-1}) --- this is going to be used for DGP
p_star = p_star./sum(p_star);
cdf_star = zeros(ns+1,1);
cdf_star(1) = 0;
for i = 1:ns
    cdf_star(i+1) = cdf_star(i) + p_star(i);
end


% (4) Infeasible optimal weighting matrix for the ASL estimator 
dP = Psi_P('Psi',P_1_star,theta,f_s,SS,AA); 
dP_dtheta = dPsi(P_1_star,theta,f_s,SS,AA);
Sigma = Sigma_PS(P_1_star,p_star(:),AA);  
W_star = inv((eye(ns*nglob)-dP)*Sigma*(eye(ns*nglob)-dP')); 


% (5) Descriptive statistics for the model
p_entry = P_1_star'*p_star;
disp('Probability of being active [firm1, firm2, firm3]=');
disp(P_1_star'*p_star);

dP = Psi_P('Psi',P_1_star,theta,f_s,SS,AA); 
dP_Lambda = Psi_P('Lambda',P_1_star,theta,f_s,SS,AA,alpha); 
dP_dtheta = dPsi(P_1_star,theta,f_s,SS,AA);
dP_dtheta = [dP_dtheta;-dP_dtheta];
Delta_P = diag(repmat(p_star,2*nglob,1)./[P_1_star(:);1-P_1_star(:)]);
Proj = dP_dtheta*inv(dP_dtheta'*Delta_P*dP_dtheta)*dP_dtheta'*Delta_P;
M = eye(2*ns*nglob) - Proj; % M_Lambda = M
M_11 = M(1:ns*nglob,1:ns*nglob);
M_12 = M(1:ns*nglob,ns*nglob+1:2*ns*nglob);
 


disp('Table 1: Dominant Eigenvalue of Psi and Lambda'); 
disp('\theta_{RN}=');disp(theta_RN);
disp('[alpha,\rho(Psi_P),\rho(Lambda_P),\rho(M*Psi_P),\rho(M*Lambda_P)]=');
disp([alpha,max(abs(eig(dP))),max(abs(eig(dP_Lambda))),max(abs(eig((M_11-M_12)*dP))),max(abs(eig((M_11-M_12)*dP_Lambda)))]);

disp('[alpha,||Psi_P||_s,||Lambda_P||_s,||M*Psi_P||_s,||M*Lambda_P||_s]=');
disp([alpha,sqrt(max(abs(eig(dP'*dP)))),max(sqrt(abs(eig(dP_Lambda'*dP_Lambda)))),max(sqrt(abs(eig(((M_11-M_12)*dP)'*((M_11-M_12)*dP))))),max(sqrt(abs(eig(((M_11-M_12)*dP_Lambda)'*(M_11-M_12)*dP_Lambda))))]);



if dummy_period2 == 1
    % Solve an approximated NPL fixed points under 2-period cycle with a
    % large number of observations
    nobs_2 = 100000;
    dat = zeros(nobs_2,2,nglob);
    x_0 = zeros(nobs_2,1);
    for j = 1:nobs_2
        u_0 = rand;
        for t = 1:ns
            if (u_0>=cdf_star(t))*(u_0<=cdf_star(t+1))
                x_0(j) = t;
                break;
            end
        end
    end
    dat(:,1,:) = repmat(x_0,1,nglob); %x_0 is common across firms
    for i = 1:nglob
        dat(:,2,i) = (rand(nobs_2,1)<P_1_star(x_0,i));
    end
    x_0 = squeeze(dat(:,1,:));
    a_1 = squeeze(dat(:,2,:));
    P_1 = P_psi_0;  
    b = theta(1:2,1);
    b_1 = b;
    b_2 = b;
    zz = zeros(nobs_2,npar_i,nglob);
    lambda = zeros(nobs_2,nglob);
    options = optimset('Display','off','GradObj','on','LargeScale','off','TolFun',1e-6,'TolX',1e-6);
    for i = 1:1000 % NPL algorithm under Psi  
        if i/2 == floor(i/2) % i is even
            P_psi_1 = P_1;
            b_1 = b;
        else %if i is odd
            P_psi_2 = P_1;
            b_2 = b; 
        end
        %   Step 1: compute(z_tilde,lambda_tilde) given P_j  
        [z_tilde,lambda_tilde] = z_lambda(P_1,f_s,SS,AA);  
        for t = 1:nglob
            for j = 1:npar_i
                zz(:,j,t) = z_tilde(x_0(:,t),j,t);
            end
            lambda(:,t) = lambda_tilde(x_0(:,t),t); 
        end
        %   Step 2: update theta_{j+1} by PML
        if i/2 == floor(i/2) % i is even
            b = b_2;
        else
            b = b_1;
        end
        b = fminunc('logit_Psi',b,options,theta,zz,lambda,a_1); 
        %   Step 3: update P_j
        theta_b = theta;
        theta_b(1:2,1) = b;
        P_1 = Psi(P_1,theta_b,f_s,SS,AA); % a standard mapping
        if abs(P_psi_1(:)-P_1(:))<convcons
            break
        end
    end
    if i>=1000
        disp('the NPL did not converge under 2-period cycle. Stopping...');
        stop
    end 
    disp('Table A.1: Dominant Eigenvalue of MPsi_P_1*MPsi_P_2 and MPsi_P_2*MPsi_P_1 under 2-period cycle'); 
    theta_1 = [b_1(1:2);theta(3:6)];
    theta_2 = [b_2(1:2);theta(3:6)];
    dP_1 = Psi_P('Psi',P_psi_1,theta_2,f_s,SS,AA);  
    dP_2 = Psi_P('Psi',P_psi_2,theta_1,f_s,SS,AA);  
    dP_dtheta_1 = dPsi(P_psi_1,theta_2,f_s,SS,AA);
    dP_dtheta_1 = [dP_dtheta_1;-dP_dtheta_1];
    dP_dtheta_2 = dPsi(P_psi_2,theta_1,f_s,SS,AA);
    dP_dtheta_2 = [dP_dtheta_2;-dP_dtheta_2];
    Delta_P_1 = diag(repmat(p_star,2*nglob,1).*[P_1_star(:);1-P_1_star(:)]./([P_psi_2(:);1-P_psi_2(:)].^2));
    Delta_P_2 = diag(repmat(p_star,2*nglob,1).*[P_1_star(:);1-P_1_star(:)]./([P_psi_1(:);1-P_psi_1(:)].^2));
    Proj_1 = dP_dtheta_1*inv(dP_dtheta_1'*Delta_P_1*dP_dtheta_1)*dP_dtheta_1'*Delta_P_1;
    Proj_2 = dP_dtheta_2*inv(dP_dtheta_2'*Delta_P_2*dP_dtheta_2)*dP_dtheta_2'*Delta_P_2;
    M1 = eye(2*ns*nglob) - Proj_1;  
    M2 = eye(2*ns*nglob) - Proj_2;  
    M1_11 = M2(1:ns*nglob,1:ns*nglob);
    M1_12 = M2(1:ns*nglob,ns*nglob+1:2*ns*nglob);
    M2_11 = M2(1:ns*nglob,1:ns*nglob);
    M2_12 = M2(1:ns*nglob,ns*nglob+1:2*ns*nglob);
    disp('\theta_{RN}=');disp(theta_RN);
    disp('\rho(MPsi_P_1*MPsi_P_2)=');
    disp('[\rho(MPsi_P_1),\rho(*MPsi_P_2),\rho(MPsi_P_1*MPsi_P_2),\rho(MPsi_P_2*MPsi_P_1)]=');
    disp( [max(abs(eig((M1_11-M1_12)*dP_1)));
        max(abs(eig((M2_11-M2_12)*dP_2)));
        max(abs(eig((M1_11-M1_12)*dP_1*(M2_11-M2_12)*dP_2)));
        max(abs(eig((M2_11-M2_12)*dP_2*(M1_11-M1_12)*dP_1)))] );
end




 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% Monte Carlo Experiment %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (1) draw a sample from the population
% (2) Estimation of theta
%   Step 0: estimate P_0
%   Step 1: compute (z_tilde,lambda_tilde) given P_j
%   Step 2: update theta_{j+1} by estimating logit
%   Step 3: update P_{j+1} = \Psi(P_j,theta_{j+1})


% Initialization
n_alg = 6; % number of algorithms 
lambda_P_iter = zeros(nsim,n_alg);
lambda_b_iter = zeros(nsim,n_alg);
number_iter = zeros(nsim,n_alg);
theta_iter = zeros(nsim,2,n_alg);
theta_2step = zeros(nsim,2,n_alg+1);
P_1_iter = zeros(nsim,ns*nglob,n_alg+1);
P_2step = zeros(nsim,ns*nglob,n_alg+1);
P_NPL_iter = zeros(nsim,ns*nglob,max_iter);
theta_NPL_iter = zeros(nsim,2,max_iter);
theta_NPL_lambda_iter = zeros(nsim,2,max_iter);
theta_NPL_q_iter = zeros(nsim,2,max_iter);
theta_AFXP_iter = zeros(nsim,2,max_iter); 
theta_RPM_iter = zeros(nsim,2,max_iter); 
theta_PS_iter = zeros(nsim,2,max_iter); 
nonconv_iter = zeros(nsim,n_alg);
size_Z = zeros(nsim,1);
iter = 1; 
% Iterations begin here
while iter <=nsim

    % (1) draw a sample from the population
    dat = zeros(nobs,2,nglob);
    x_0 = zeros(nobs,1);
    for j = 1:nobs
        u_0 = rand;
        for t = 1:ns
            if (u_0>=cdf_star(t))*(u_0<=cdf_star(t+1))
                x_0(j) = t;
                break;
            end
        end
    end
    dat(:,1,:) = repmat(x_0,1,nglob); %x_0 is common across firms
    for i = 1:nglob
        dat(:,2,i) = (rand(nobs,1)<P_1_star(x_0,i));
    end
    x_0 = squeeze(dat(:,1,:));
    a_1 = squeeze(dat(:,2,:));

    % (2) Estimation of (P,theta)
    % (2.0) An initial estimator: P_hat
    P_x_hat = zeros(ns,1);
    for i = 1:ns
        P_x_hat(i) = sum(x_0(:,1)==i)/nobs;
    end
    
    P_x_hat = max(min(P_x_hat,1-1e-4),1e-4); % Frequency estimator for P_x
    if spec_P0 == 'LG'
        P_hat = zeros(ns,nglob);
        lnS = zeros(nstate,na);
        lnS(:,:) = log(SS)*ones(1,na);
        lnS = lnS(:);
        s_0 = zeros(nobs,1);
        s_0(:,1) = lnS(dat(:,1,1));
        a_0 = ones(nstate,1)*(1:1:na);
        a_0 = a_0(:);
        a_0 = a_0(dat(:,1,1));
        b = zeros((2+nglob)*nglob,1);
        options = optimset('Display','off','GradObj','on','LargeScale','off','TolFun',1e-6,'TolX',1e-6);
        [b,v1] = fminunc('clogit',b,options,s_0,a_0,a_1,AA);
        P_hat = zeros(ns,nglob);
        b = reshape(b,2+nglob,nglob);
        a_0 = ones(nstate,1)*(1:1:na);
        a_0 = a_0(:);
        for i = 1:nglob
            P_hat(:,i) = 1./(1+exp(- b(1,i)- b(2,i)*lnS - AA(a_0,:)*b(3:2+nglob,i)));
        end
        P_hat = max(min(P_hat,1-1e-4),1e-4);
    elseif spec_P0 == 'FQ'
        P_hat = zeros(ns,nglob);
        mean_a_1 = mean(a_1)'; % nglob x 1
        for i = 1:ns
            for j = 1:nglob
                index = (x_0(:,j)==i); %
                if sum(index) > 0
                    P_hat(i,j) = sum(a_1(index,j))/sum(index);
                elseif sum(index)==0
                    P_hat(i,j) = mean_a_1(j);
                end
            end
        end
        P_hat = max(min(P_hat,1-1e-4),1e-4);
    elseif spec_P0 == 'TR'
        P_hat = P_1_star;
    elseif spec_P0 == 'UN'
        P_hat = 0.5*zeros(ns,nglob);
    elseif spec_P0 == 'RD'
        P_hat = rand(ns,nglob);
    else
        disp('Specify an initial estimator for P_0. Stopping...');
        stop
    end
    P_1_iter(iter,:,n_alg+1) = P_hat(:); % an initial estimate

    % (2.1) Estimation of (P,theta) by the NPL
    %   Step 0: estimate P_1 and  compute(z_tilde,lambda_tilde) given P_0
    P_1 = P_hat;  % an initial estimate
    zz = zeros(nobs,npar_i,nglob);
    lambda = zeros(nobs,nglob);
    b_iter = zeros(2,max_iter);
    P_iter = zeros(ns*nglob,max_iter); 
    b = theta(1:2,1); %zeros(2,1); % an initial parameter value
    options = optimset('Display','off','GradObj','on','LargeScale','off','TolFun',1e-6,'TolX',1e-6);
    for i = 1:max_iter
        b_iter(:,i) = b;
        P_iter(:,i) = P_1(:);
        b_j = b;
        P_j = P_1;
        %   Step 1: compute(z_tilde,lambda_tilde) given P_j
        %   (z_tilde, lambda_tilde) does not depend on theta
        %   see AM07, p. 12-13.
        P = zeros(nobs,nglob);
        [z_tilde,lambda_tilde] = z_lambda(P_1,f_s,SS,AA); 
        for t = 1:nglob
            for j = 1:npar_i
                zz(:,j,t) = z_tilde(x_0(:,t),j,t);
            end
            lambda(:,t) = lambda_tilde(x_0(:,t),t); 
        end
        %   Step 2: update theta_{j+1} by PML
        b = fminunc('logit_Psi',theta(1:2,1),options,theta,zz,lambda,a_1);
        theta_NPL_iter(iter,:,i) = b';
        %   Step 3: update P_j
        theta_b = theta;
        theta_b(1:2,1) = b;
        P_1 = Psi(P_1,theta_b,f_s,SS,AA); % a standard mapping
        P_NPL_iter(iter,:,i) = P_1(:)';
        if i == 1
            theta_2step(iter,:,1)=b'; % two-step estimator
            P_2step(iter,:,1) = P_1(:)';
        end
        if (sum(abs(P_j(:)-P_1(:)))+sum(abs(b_j(:)-b(:))))<convcons
            conviter = i;
            for j = i:max_iter
                theta_NPL_iter(iter,:,j) = b';
            end
            break
        end
    end
    if i == max_iter % then the NPL algorithm did not converge
        disp('P does not converge in the NPL');
        nonconv_iter(iter,1) = nonconv_iter(iter,1) + 1;
    end  
    theta_iter(iter,:,1) = b';
    P_1_iter(iter,:,1) = P_1(:)';
% 
% 
%     % (2.2) Estimation of (P,theta) by the Lambda
%     %   Step 0: estimate P_1 and  compute(z_tilde,lambda_tilde) given P_0
%     P_1 = P_hat;   % an initial estimate
%     zz = zeros(nobs,npar_i,nglob);
%     lambda = zeros(nobs,nglob);
%     b_iter = zeros(2,max_iter);
%     P_iter = zeros(ns*nglob,max_iter);
%     b = theta(1:2,1); %zeros(2,1); % an initial parameter value 
%     options = optimset('Display','off','GradObj','off','LargeScale','off','TolFun',1e-6,'TolX',1e-6);
%     for i = 1:max_iter
%         b_iter(:,i) = b;
%         P_iter(:,i) = P_1(:);
%         b_j = b;
%         P_j = P_1;
%         %   Step 1: update theta_{j+1}
%         %   Step 1(a): compute(z_tilde,lambda_tilde) given P_j
%         %   (z_tilde, lambda_tilde) does not depend on theta
%         P = zeros(nobs,nglob);
%         [z_tilde,lambda_tilde] = z_lambda(P_1,f_s,SS,AA);
%         for t = 1:nglob
%             for j = 1:npar_i
%                 zz(:,j,t) = z_tilde(x_0(:,t),j,t);
%             end
%             lambda(:,t) = lambda_tilde(x_0(:,t),t);
%             P(:,t) = P_1(x_0(:,t),t);
%         end
%         %   Step 1(b): PML
%         b = fminunc('logit_Lambda',theta(1:2,1),options,theta,zz,lambda,a_1,P,alpha);
%         theta_NPL_lambda_iter(iter,:,i) = b';
%         %   Step 2: update P_j
%         theta_b = theta;
%         theta_b(1:2,1) = b;
%         P_1 = Lambda(P_1,theta_b,f_s,SS,AA,alpha); 
%         if i == 1
%             theta_2step(iter,:,2)=b'; % two-step estimator
%             P_2step(iter,:,2) = P_1(:)';
%         end
%         if (sum(abs(P_j(:)-P_1(:)))+sum(abs(b_j(:)-b(:))))<convcons
%             conviter = i;
%             for j = i:max_iter
%                 theta_NPL_lambda_iter(iter,:,j) = b';
%             end
%             break
%         end
%     end
%     if i == max_iter % then the NPL algorithm did not converge
%         disp('P does not converge in the NPL Lambda');
%         nonconv_iter(iter,2) = nonconv_iter(iter,2) + 1;
%     end 
%     theta_iter(iter,:,2) = b';
%     P_1_iter(iter,:,2) = P_1(:)';
% 
% 
%     
%     % (2.3) Estimation of (P,theta) by the RPM (based on p.1109 of SK)
%     %   Step 0: estimate P_1 and compute(z_tilde,lambda_tilde) given P_0
%     zz = zeros(nobs,npar_i,nglob);
%     lambda = zeros(nobs,nglob);
%     b_iter = zeros(2,max_iter);
%     P_iter = zeros(ns*nglob,max_iter);
%     b = zeros(2,1); % an initial parameter value
%     conviter = 0;
%     options = optimset('Display','off','GradObj','on','LargeScale','off','TolFun',1e-6,'TolX',1e-6);
%     for i = 1:max_iter
%         b_j = b;
%         P_j = P_1;
%         % Step 1: update theta
%         if i <= 1
%             % Step 0: initialization
%             b = squeeze(theta_NPL_iter(iter,:,i))';
%             P_1 = P_hat;  % start from an initial estimator
%             theta_b = theta;
%             theta_b(1:2,1) = b;
%             P_1 = Psi(P_1,theta_b,f_s,SS,AA); % a standard mapping 
%             dPsi_dp = Psi_P('Psi',P_1,theta_b,f_s,SS,AA);  
%             dPsi_dp = real(dPsi_dp);
%             Z = basis_RPM(dPsi_dp,delta_RPM);
%             Proj_P = Z*Z';
%             Proj_Q = eye(ns*nglob) - Z*Z';
%             inv_I_f = inv(eye(size(Z,2))-Z'*dPsi_dp*Z);
%         else %if i>1
%             [z_tilde,lambda_tilde] = z_lambda(P_1,f_s,SS,AA);
%             options = optimset('Display','off','GradObj','off','LargeScale','off','TolFun',1e-6,'TolX',1e-6);
%             b = fminunc('logit_RPM',theta(1:2),options,theta,dat,z_tilde,lambda_tilde,a_1,P_1,Z,inv_I_f,Proj_Q); 
%             % Step 2: update P
%             theta_b0 = theta_b;
%             theta_b = theta;
%             theta_b(1:2,1) = b;   
%             P_1 = RPM(P_1,theta_b,f_s,SS,AA,inv_I_f,Z);  
%             % Step 3: update inv_I_f, Proj_P, Proj_Q 
%             if isinteger(i/10) % full updating every 10 iterations
%                 dPsi_dp = Psi_P('Psi',P_1,theta_b,f_s,SS,AA);  
%                 %dPsi_dp = real(dPsi_dp);
%                 Z = basis_RPM(dPsi_dp,delta_RPM); 
%                 Proj_P = Z*Z';
%                 Proj_Q = eye(ns*nglob) - Z*Z';
%                 inv_I_f = inv(eye(size(Z,2))-Z'*dPsi_dp*Z);
%             else % Z is updated without changing the size of Z
%                 %(a) Psi_P * Z
%                 h = eps^(1/3);
%                 P_j = Psi(P_1,theta_b,f_s,SS,AA); 
%                 Psi_P_Z = zeros(size(Z));
%                 for t = 1:size(Z,2)
%                     P_z = Psi(P_1+h*reshape(Z(:,t),ns,nglob),theta_b,f_s,SS,AA); % a standard mapping
%                     Psi_P_Z(:,t) = (1/h)*(P_z(:)-P_j(:));
%                 end
%                 %inv_I_f = inv(eye(size(Z,2))-Z'*Psi_P_Z); 
%                 % (b) Update Z: power iteration step 
%                 Z = orth(Psi_P_Z); 
%                 % (c) update inv_I_f_P, Proj_P, Proj_Q
%                 h = eps^(1/3); 
%                 Psi_P_Z = zeros(size(Z)); 
%                 for t = 1:size(Z,2) 
%                     P_z = Psi(P_1+h*reshape(Z(:,t),ns,nglob),theta_b,f_s,SS,AA); % a standard mapping
%                     Psi_P_Z(:,t) = (1/h)*(P_z(:)-P_j(:)); 
%                 end
%                 Proj_P = Z*Z'; 
%                 Proj_Q = eye(ns*nglob) - Z*Z'; 
%                 inv_I_f = inv(eye(size(Z,2))-Z'*Psi_P_Z); 
%             end 
%         end 
%         theta_RPM_iter(iter,:,i) = b'; 
%         if sum(abs(theta(1:2)-b(:)))>1000
%             disp('Divergence in b');
%             stop
%         end
%         if i == 2
%             theta_2step(iter,:,3)= b'; % two-step estimator
%             P_2step(iter,:,3) = P_1(:)';
%         end
%         b_iter(:,i) = b;
%         P_iter(:,i) = P_1(:);
%         if (sum(abs(P_j(:)-P_1(:)))+sum(abs(b_j(:)-b(:)))) < convcons
%             conviter = i;
%             for j = i:max_iter
%                 theta_RPM_iter(iter,:,j) = b';
%             end
%             break
%         end
%     end
%     if i == max_iter
%         disp('P does not converge in the RPM'); 
%         nonconv_iter(iter,3) = nonconv_iter(iter,3) + 1;
%     end 
%     theta_iter(iter,:,3) = b';
%     P_1_iter(iter,:,3) = P_1(:)';
%     size_Z(iter,1) = size(Z,2);
%     
%     % (2.4) Estimation of (P,theta) by the q-NPL 
%     %   Step 0: estimate P_1 and  compute(z_tilde,lambda_tilde) given P_0
%     P_1 = P_hat;   % an initial estimate
%     zz = zeros(nobs,npar_i,nglob);
%     lambda = zeros(nobs,nglob);
%     b_iter = zeros(2,max_iter);
%     P_iter = zeros(ns*nglob,max_iter);
%     b = theta(1:2,1); %zeros(2,1); % an initial parameter value
%     options = optimset('Display','off','GradObj','off','LargeScale','off','TolFun',1e-6,'TolX',1e-6);
%     for i = 1:max_iter
%         b_iter(:,i) = b;
%         P_iter(:,i) = P_1(:);
%         b_j = b;
%         P_j = P_1;
%         %   Step 1: update theta_{j+1}
%         if i <= 1
%             b = squeeze(theta_NPL_iter(iter,:,i))'; % PML estimator
%         else %if i>1 --- approximate q-NPL
%             dP_dtheta = Psi_theta('Lambda_q',np,P_1,theta_b,f_s,SS,AA,alpha,q);  
%             dP_dtheta = real(dP_dtheta);
%             [z_tilde,lambda_tilde] = z_lambda(P_1,f_s,SS,AA);
%             options = optimset('Display','off','GradObj','off','LargeScale','off','TolFun',1e-6,'TolX',1e-6);
%             b = fminunc('logit_P_theta',theta(1:2),options,theta,dat,z_tilde,lambda_tilde,a_1,dP_dtheta,P_1,b_j);
%         end
%         theta_NPL_q_iter(iter,:,i) = b';
%         %   Step 2: update P_j
%         theta_b = theta;
%         theta_b(1:2,1) = b; 
%         P_1 = Lambda_q(P_1,theta_b,f_s,SS,AA,alpha,q); 
%         if i == 2
%             theta_2step(iter,:,4)= b'; % two-step estimator
%             P_2step(iter,:,4) = P_1(:)';
%         end
%         if (sum(abs(P_j(:)-P_1(:)))+sum(abs(b_j(:)-b(:))))<convcons
%             conviter = i;
%             for j = i:max_iter
%                 theta_NPL_q_iter(iter,:,j) = b';
%             end
%             break
%         end
%     end
%     if i == max_iter % then the q-NPL algorithm did not converge
%         disp('P does not converge in the q-NPL '); 
%         nonconv_iter(iter,4) = nonconv_iter(iter,4) + 1;
%     end 
%     theta_iter(iter,:,4) = b';
%     P_1_iter(iter,:,4) = P_1(:)';
% 
% 
%     % (2.5) Estimation of (P,theta) by the AFXP using Lambda^q to update P
%     %   Step 0: estimate P_1 and compute(z_tilde,lambda_tilde) given P_0
%     P_1 = P_hat;  % start from uniform
%     zz = zeros(nobs,npar_i,nglob);
%     lambda = zeros(nobs,nglob);
%     b_iter = zeros(2,max_iter);
%     P_iter = zeros(ns*nglob,max_iter);
%     b = zeros(2,1); % an initial parameter value
%     conviter = 0;
%     options = optimset('Display','off','GradObj','on','LargeScale','off','TolFun',1e-6,'TolX',1e-6);
%     for i = 1:max_iter
%         b_iter(:,i) = b;
%         P_iter(:,i) = P_1(:);
%         b_j = b;
%         P_j = P_1;
%         %   Step 1: update theta_{j+1}
%         if i <= 1 % an initial estimator is the NPL estimator
%             alpha_i = alpha;
%             b = squeeze(theta_NPL_iter(iter,:,i))'; 
%         else %if i>1 then use approximate fixed point alogrithm
%             dP_dtheta = dPsi(P_1,theta_b,f_s,SS,AA);
%             dPsi_dP = Psi_P('Psi',P_1,theta_b,f_s,SS,AA);
%             dPsi_dP = real(dPsi_dP);
%             dP_dtheta = (eye(ns*nglob)-dPsi_dP)\dP_dtheta;
%             [z_tilde,lambda_tilde] = z_lambda(P_1,f_s,SS,AA);
%             options = optimset('Display','off','GradObj','off','LargeScale','off','TolFun',1e-6,'TolX',1e-6);
%             b = fminunc('logit_P_theta',b_j,options,theta,dat,z_tilde,lambda_tilde,a_1,dP_dtheta,P_1,b_j);
%         end
%         theta_AFXP_iter(iter,:,i) = b';
%         if sum(abs(theta(1:2)-b(:)))>100 % safeguard against divergence
%             disp('Divergence in b');
%             stop
%         end
%         %   Step 2: update P_j 
%         theta_b = theta;
%         theta_b(1:2,1) = b;
%         P_1 = Lambda_q(P_1,theta_b,f_s,SS,AA,alpha,q); % fixed point 
%         if i == 2
%             theta_2step(iter,:,5)= b'; % two-step estimator
%             P_2step(iter,:,5) = P_1(:)';
%         end
%         if (sum(abs(P_j(:)-P_1(:)))+sum(abs(b_j(:)-b(:)))) < convcons
%             conviter = i;
%             for j = i:max_iter
%                 theta_AFXP_iter(iter,:,j) = b';
%             end
%             break
%         end
%     end
%     if i == max_iter
%         disp('P does not converge in the AFXP'); 
%         nonconv_iter(iter,5) = nonconv_iter(iter,5) + 1;
%     end 
%     theta_iter(iter,:,5) = b';
%     P_1_iter(iter,:,5) = P_1(:)'; 
    
    
    % (2.6) Estimation of (P,theta) by the Asymptotic LS estimator by PS
    %   Step 0: estimate P_1 and compute(z_tilde,lambda_tilde) given P_0
    P_1 = P_hat;  % start from uniform
    zz = zeros(nobs,npar_i,nglob);
    lambda = zeros(nobs,nglob);
    b_iter = zeros(2,max_iter);
    P_iter = zeros(ns*nglob,max_iter);
    b = zeros(2,1); % an initial parameter value
    conviter = 0;
    options = optimset('Display','off','GradObj','off','LargeScale','off','TolFun',1e-6,'TolX',1e-6);
    for i = 1:3
        b_iter(:,i) = b;
        P_iter(:,i) = P_1(:);
        b_j = b;
        P_j = P_1;
        %   Step 0: weighting matrix
        if i == 1  % using identity weighting matrix
            W = eye(ns*nglob);
        elseif i == 2; % using efficient weighting matrix
            dPsi_dP = Psi_P('Psi',P_hat,theta_b,f_s,SS,AA);
            Sigma = Sigma_PS(P_hat,P_x_hat,AA);
            W = inv((eye(ns*nglob,ns*nglob)-dPsi_dP)*Sigma*(eye(ns*nglob,ns*nglob)-dPsi_dP)');
        else %if i == 3;
            W = W_star;
        end
        %   Step 1: update theta_{j+1}
        b = fminunc('ALS',theta(1:2),options,P_hat,theta,f_s,SS,AA,W);   
        theta_PS_iter(iter,:,i) = b';
%        if sum(abs(theta(1:2)-b(:)))>100 % safeguard against divergence
%            disp('Divergence in b');
%            stop
%        end
        %   Step 2: update P_j  
        theta_b = theta;
        theta_b(1:2,1) = b;
        P_1 = Psi(P_hat,theta_b,f_s,SS,AA); % a standard mapping 
        if i == 1
            theta_2step(iter,:,6)= b';  
            P_2step(iter,:,6) = P_1(:)';
        elseif i == 2 
            theta_2step(iter,:,7)= b';  
            P_2step(iter,:,7) = P_1(:)';
        else %if i == 3
            for j = i:max_iter
                theta_PS_iter(iter,:,j) = b';
            end
            break
        end
    end 
    theta_iter(iter,:,6) = b';
    P_1_iter(iter,:,6) = P_1(:)'; 
    
    iter = iter+1;

    if iter/10 == floor(iter/10)
        disp(iter);
    end

    %     if conviter == max_iter
    %         disp('conviter == max_iter');
    %         sample_skip = sample_skip + 1;
    %         disp('b=');disp(b);
    %     else
    %         iter = iter +1;
    %     end


end



% The summary statistics from simulations:

diary('mpe_sim15_result');
diary on

% Parameter values
%disp{' ');
disp('(A) The Simulation Set-up');
%disp{' ');
disp('the number of simulated samples=');disp(nsim);
disp('the number of observations for each sample=');disp(nobs);
disp('true value of theta=');disp(theta');
disp('theta_RN=');disp(theta_RN);
disp(' ');
disp('The initial estimator for P=');disp(spec_P0);
disp(' ');
disp('The number of Psi iteration for q-NPL=');disp(q);
disp(' ');
disp('the number of global firms=');disp(nglob);
disp('the number of S states =');disp(nstate);
disp(' ');
disp(' ');
disp('The properties of $\Psi$'); 
disp('A fraction of samples that did not converge under [NPL;Lambda;RPM;q-NPL;AFXP]');
disp(sum(nonconv_iter(:,1:5),1)./nsim); 
disp(' ');
disp('The value of alpha=');
disp(alpha);
disp(' ');
disp('(B) The Simulation Results');
disp(' '); 
disp('(B.1) Bias and MSE for theta');
disp(' ');
rowlabels = ['NPL with $\Psi$        ';
             'NPL with $\Lambda$     ';
             'RPM ($\delta=0.5$)     ';
             'q-NPL with $\Lambda^q$ '; 
             'q-AFXP with $\Lambda^q$';
             'Infeasible LS-E by PS  '];
disp('Row labels are:');
disp(' ');
disp(rowlabels);
disp('  ');
bias = zeros(n_alg,2);
variance = zeros(n_alg,2);
mse = zeros(n_alg,2);
bias_P = zeros(n_alg+1,ns*nglob);
variance_P = zeros(n_alg+1,ns*nglob);
mse_P = zeros(n_alg+1,ns*nglob);
for i = 1:n_alg
    bias(i,:) = mean(squeeze(theta_iter(:,:,i))-ones(nsim,1)*theta(1:2,1)');
    variance(i,:) = mean((squeeze(theta_iter(:,:,i))-ones(nsim,1)*mean(squeeze(theta_iter(:,:,i)))).^2);
    bias_P(i,:) = mean(squeeze(P_1_iter(:,:,i))-ones(nsim,1)*P_1_star(:)');
    variance_P(i,:) = mean((squeeze(P_1_iter(:,:,i))-ones(nsim,1)*mean(squeeze(P_1_iter(:,:,i)))).^2);
end
i=n_alg+1; 
bias_P(i,:) = mean(squeeze(P_1_iter(:,:,i))-ones(nsim,1)*P_1_star(:)');
variance_P(i,:) = mean((squeeze(P_1_iter(:,:,i))-ones(nsim,1)*mean(squeeze(P_1_iter(:,:,i)))).^2);
mse = bias.^2 + variance;
mse_P = bias_P.^2 + variance_P.^2;
columnlabels=['    Bias  ','    RMSE  '];
for j = 1:2
    if j == 1
        disp('\theta_{RS}');
    else
        disp('\theta_{RN}');
    end  
    disp(columnlabels);
    disp([bias(:,j),sqrt(mse(:,j))]); 
end  
disp('100*Integrated BIAS and MSE for P (the last row is frequency estimator)=');
disp([100*mean(bias_P')',100*sqrt(mean(mse_P')')]); 

disp('Two-step estimators');
rowlabels2 = ['PML with $\Psi$        ';
             'PML with $\Lambda$     ';
             '2-PML with RPM         ';
             '2-PML with $\Lambda^q$ '; 
             '2-PML with q-AFXP      ';
             'LS-I by PS             ';
             'LS-E by PS             '];
disp('Row labels are:');
disp(' ');
disp(rowlabels2);
disp('  ');
mse2 = zeros(n_alg+1,2);
bias2_P = zeros(n_alg+1,ns*nglob);
variance2_P = zeros(n_alg+1,ns*nglob);
mse2_P = zeros(n_alg+1,ns*nglob);
for i = 1:n_alg+1
    bias2(i,:) = mean(squeeze(theta_2step(:,:,i))-ones(nsim,1)*theta(1:2,1)');
    variance2(i,:) = mean((squeeze(theta_2step(:,:,i))-ones(nsim,1)*mean(squeeze(theta_2step(:,:,i)))).^2);
    bias2_P(i,:) = mean(squeeze(P_2step(:,:,i))-ones(nsim,1)*P_1_star(:)');
    variance2_P(i,:) = mean((squeeze(P_2step(:,:,i))-ones(nsim,1)*mean(squeeze(P_2step(:,:,i)))).^2);
end
mse2 = bias2.^2 + variance2;
mse2_P = bias2_P.^2 + variance2_P.^2;
for j = 1:2
    if j == 1
        disp('\theta_{RS}');
    else
        disp('\theta_{RN}');
    end
    disp(columnlabels);
    disp([bias2(:,j),sqrt(mse2(:,j))]);
end
disp('100*Integrated BIAS and MSE of P for 2 or 3 step estimators=');
disp([100*mean(bias2_P')',100*sqrt(mean(mse2_P')')]);

disp('(B.2) Bias and MSE for iter = 1,2,...,30');
if max_iter >= 30
    max_iter = 30;
end
bias_NPL = zeros(max_iter,2);
variance_NPL = zeros(max_iter,2);
bias_Lambda = zeros(max_iter,2);
variance_Lambda = zeros(max_iter,2);
bias_RPM = zeros(max_iter,2); 
variance_RPM = zeros(max_iter,2);
bias_NPL_q = zeros(max_iter,2);
variance_NPL_q = zeros(max_iter,2);
bias_AFXP = zeros(max_iter,2);
variance_AFXP = zeros(max_iter,2); 
for i = 1:max_iter
    bias_NPL(i,:) = mean(squeeze(theta_NPL_iter(:,:,i))-ones(nsim,1)*theta(1:2,1)');
    variance_NPL(i,:) = mean((squeeze(theta_NPL_iter(:,:,i))-ones(nsim,1)*mean(squeeze(theta_NPL_iter(:,:,i)))).^2);
    bias_Lambda(i,:) = mean(squeeze(theta_NPL_lambda_iter(:,:,i))-ones(nsim,1)*theta(1:2,1)');
    variance_Lambda(i,:) = mean((squeeze(theta_NPL_lambda_iter(:,:,i))-ones(nsim,1)*mean(squeeze(theta_NPL_lambda_iter(:,:,i)))).^2);
    bias_RPM(i,:) = mean(squeeze(theta_RPM_iter(:,:,i))-ones(nsim,1)*theta(1:2,1)');
    variance_RPM(i,:) = mean((squeeze(theta_RPM_iter(:,:,i))-ones(nsim,1)*mean(squeeze(theta_RPM_iter(:,:,i)))).^2);
    bias_NPL_q(i,:) = mean(squeeze(theta_NPL_q_iter(:,:,i))-ones(nsim,1)*theta(1:2,1)');
    variance_NPL_q(i,:) = mean((squeeze(theta_NPL_q_iter(:,:,i))-ones(nsim,1)*mean(squeeze(theta_NPL_q_iter(:,:,i)))).^2);
    bias_AFXP(i,:) = mean(squeeze(theta_AFXP_iter(:,:,i))-ones(nsim,1)*theta(1:2,1)');
    variance_AFXP(i,:) = mean((squeeze(theta_AFXP_iter(:,:,i))-ones(nsim,1)*mean(squeeze(theta_AFXP_iter(:,:,i)))).^2);  
end
mse_NPL = bias_NPL.^2 + variance_NPL;
mse_Lambda = bias_Lambda.^2 + variance_Lambda;
mse_NPL_q = bias_NPL_q.^2 + variance_NPL_q;
mse_AFXP = bias_AFXP.^2 + variance_AFXP; 
mse_RPM = bias_RPM.^2 + variance_RPM; 
for j = 1:2
    if j == 1
        disp('\theta_{RS}');
    else
        disp('\theta_{RN}');
    end
    disp('bias and sqrt(mse)');
    disp('[bias_NPL(:,j),sqrt(mse_NPL(:,j))]=');
    disp([bias_NPL(:,j),sqrt(mse_NPL(:,j))]);
    disp('[bias_Lambda(:,j),sqrt(mse_Lambda(:,j))]=');
    disp([bias_Lambda(:,j),sqrt(mse_Lambda(:,j))]);
    disp('[bias_RPM(:,j),sqrt(mse_RPM(:,j))]=');
    disp([bias_RPM(:,j),sqrt(mse_RPM(:,j))]);
    disp('[bias_NPL_q(:,j),sqrt(mse_NPL_q(:,j))]=');
    disp([bias_NPL_q(:,j),sqrt(mse_NPL_q(:,j))]);
    disp('[bias_AFXP(:,j),sqrt(mse_AFXP(:,j))]=');
    disp([bias_AFXP(:,j),sqrt(mse_AFXP(:,j))]);  
end 

disp('(B.3) Possible Convergence to Limit Cycle for NPL'); 
P_diff_NPL = P_NPL_iter(:,:,2:max_iter+1)-P_NPL_iter(:,:,1:max_iter);
P_diff_2_NPL = P_NPL_iter(:,:,3:max_iter+2)-P_NPL_iter(:,:,1:max_iter);
theta_diff_NPL = theta_NPL_iter(:,:,2:max_iter+1)-theta_NPL_iter(:,:,1:max_iter);
theta_diff_2_NPL = theta_NPL_iter(:,:,3:max_iter+2)-theta_NPL_iter(:,:,1:max_iter);
sd_diff_NPL = zeros(max_iter,2);
sd_diff_2_NPL = zeros(max_iter,2);
sd_P_diff_NPL = zeros(max_iter,1);
sd_P_diff_2_NPL = zeros(max_iter,1);
for i = 1:max_iter
    sd_diff_NPL(i,:) = sqrt(mean(squeeze(theta_diff_NPL(:,:,i)).^2));
    sd_diff_2_NPL(i,:) = sqrt(mean(squeeze(theta_diff_2_NPL(:,:,i)).^2));
    sd_P_diff_NPL(i) = sqrt(mean(mean(squeeze(P_diff_NPL(:,:,i)).^2)));
    sd_P_diff_2_NPL(i) = sqrt(mean(mean(squeeze(P_diff_2_NPL(:,:,i)).^2)));
end 
disp('  ');
disp('$\sqrt{MSE}$ of (\hat\theta_{j+1}-\hat\theta_{j}) for j = 1,2,...,30:');
disp('$\theta_{RS}$ $\theta_{RN}$')
disp(sd_diff_NPL);
disp('  ');
disp('$\sqrt{MSE}$ of (\hat\theta_{j+2}-\hat\theta_{j}) for j = 1,2,...,30:');
disp('$\theta_{RS}$ $\theta_{RN}$')
disp(sd_diff_2_NPL); 
disp('  ');  
  
disp('$\sqrt{MSE}$ of (\hat P_{j+1}-\hat P_{j}) for j = 1,2,...,30:');
disp(sd_P_diff_NPL);
disp('  ');
disp('$\sqrt{MSE}$ of (\hat P_{j+2}-\hat P_{j}) for j = 1,2,...,30:');
disp(sd_P_diff_2_NPL); 
disp('  ');  
 

disp('size(Z)=');disp(mean(size_Z,1));
disp('delta_RPM=');disp(delta_RPM); 


diary off

return


















