% mpe_population.m 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% This program  (1) attempts to find all the fixed points for Psi mapping
%               (2) attempts to find all the population NPL fixed points 
%               (3) examines the convergence of the population NPL algorithm to a NPL fixed point or a 2-period cycle 
% for the paper ``Sequential Estimation of Structural Models with a Fixed Point 
% Constraint'' by Hiroyuki Kasahara and Katsumi Shimotsu 
% 
% Reference: 
% Aguirregabiria and Mira (2007) Econometrica paper 
%
% Written by Hiroyuki Kasahara, University of British Columbia 
% Last Updated: July 2011 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%   The following m-files are called by this program    %%%%  
% % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% dPsi_theta.m      computes the derivative of Psi with respect to theta 
% dPsi_P.m          computes the derivative of Psi with respect to P
% eig_Lambda        computes the dominant eigenvalue of Lambda=\Psi^{\alpha}P^{1-\alpha} 
% f_phi_population.m    computes the value of phi_0(P)-P  
% f_phi_population_NAG.m    computes the value of phi_0(P)-P for the nonlinear eq solvers c05nb
% Lambda.m          comptues the mapping with weighted average Lambda=\Psi^{\alpha}P^{1-\alpha}
% logit_Psi_population.m    computes the pseudo loglikelihood in population
% Psi.m             computes the mapping Psi 
% Psi_fp.m          computes Psi and the derivative of Psi w.r.t P 
% Psi_fp_NAG.m      computes the mapping Psi for the nonlinear eq solvers c05nb                              
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

% initialization

clear all; 
s = RandStream('mt19937ar','Seed',1); %seed random generator to ensure same results every time for benchmark purposes 
RandStream.setGlobalStream(s);
%randn('state',[89234242; 32475543]); % seed for random normal variates 
%rand('state',89234241); % seed for random uniform variates
 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%% SPECIFICATION %%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global theta f_s SS AA b_j p_star P_1_star 

% constant
eulerc = 0.5772;
convcrit = 1000;
convcons = sqrt(1.1102e-16); % 1.1102e-016 is machine precision
max_iter = 5000; % the number of iterations for NPL algorithm

% Set the values for Monte Carlo experiments
nsim = 1000; % the number of simulations
q = 4; % the number of Psi iterations in defining Psi_tilde_q
np = 2; % the number of parameters estimated 

% Model's parameter
nglob = 3; % the number of global firms
nstate = 3; % the number of market sizes
npar1 = 3; % the number of parameters that are common across firms
npar2 = nglob*1; % the number of parameters that are different across firms
npar = npar1 + npar2;
npar_i = 4;
na = 2^nglob;
ns = nstate*na; % = |X|
sigma = 1;

% parameter theta4
theta = zeros(npar,1); %specify the parameter values:
theta_RS = 1;
theta_RN = 2; % strategic substitutability
theta_EC = 1; 
theta_FC(:,1) = [1.0;0.9;0.8];
theta(:,1) = [theta_RS;theta_RN;theta_EC;theta_FC];

% transition function for S_{mt}
SS = zeros(nstate,1);
f_s = zeros(nstate,nstate);  % transition function for S_mt 
SS(:,1) = 2*[1;3;5];
f_s(:,:) = [0.8,0.2,0.0;0.2,0.6,0.2;0.0,0.2,0.8];  
% AA(x_t,i) is the value of a_{i,t-1} given x_t = (a_{1,t-1},a_{2,t-1},a_{3,t-1})

AA = zeros(na,nglob); 
for i_1 = 0:1
    for i_2 = 0:1
        for i_3 = 0:1
            i = i_1*4 + i_2*2 + i_3+1;
            AA(i,:) = [i_1,i_2,i_3];
        end
    end
end 
alpha_0 = 0.7;
%delta_RPM = 0.5;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%     Solving the fixed point    %%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (1) (a) compute the fixed point under \Psi, 
%     (b) compute the fixed point under \Lambda = \Psi^{alpha P^{1-alpha} 
%     (c) compute the optimal alpha
% (2) obtain the steady state distribution p^*(S_t,a_{t-1}) 
% (3) Descriptive statistics for the model 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 

% (1)-(a) Compute the fixed point under \Psi
P_1_star = 0.5*ones(ns,nglob);
Psi_conviter = zeros(2,1);
for j = 1:10000
    P = P_1_star;
    P_1_star = Psi(P,theta,f_s,SS,AA);
    if sum(abs(P(:)-P_1_star(:)))<convcons
        Psi_conviter(1) = j;
        break
    end
end
% dPsi_P1 = Psi_P('Psi',P_1_star,theta,f_s,SS,AA);
% dPsi_P2 = dPsi_P(P_1_star,theta,f_s,SS,AA);
% [dPsi_P1(:),dPsi_P2(:)]
% sum(abs(dPsi_P1(:)-dPsi_P2(:)))

dummy_period2 = 0;

if j>=10000 
    disp('P does not converge under Psi after 10000 iterations.'); 
    P_2 = Psi(P_1_star,theta,f_s,SS,AA);  
    P_3 = Psi(P_2,theta,f_s,SS,AA);  
    P_4 = Psi(P_3,theta,f_s,SS,AA);  
    P_5 = Psi(P_4,theta,f_s,SS,AA);  
    disp('The followings are six sequences, P_j, P_{j+1},..., P_{j+4}'); 
    disp('[P_j,P_{j+2},P_{j+4}]=');disp([P,P_2,P_4]); 
    disp('[P_{j+1},P_{j+3},P_{j+5}]=');disp([P_1_star,P_3,P_5]);  
    P_psi_0 = P; 
    P_psi_1 = P_1_star; 
    P_psi_2 = P_2; 
    if max(abs(P(:)-P_2(:)))<convcons 
        disp('The sequence \{P_j\} generated from iterating P_j = Psi(theta_j,P_{j-1}) converges to a period-2 cycle'); 
        disp('sqrt(mean_i |P_j(i)-P_{j+1}(i)|^2) at convergence ='); disp(sqrt(mean( (P_2(:)-P_3(:)).^2 ))); 
        disp('sqrt(mean_i |P_j(i)-P_{j+2}(i)|^2) at convergence ='); disp(sqrt(mean( (P(:)-P_2(:)).^2 ))); 
        disp(' '); 
        dummy_period2 = 1; 
    end 
end
   
% (1)-(b) Compute the fixed point under Lambda mapping 
P_1_star = 0.5*ones(ns,nglob); 
P = P_1_star; 
for j = 1:10000 
    P = P_1_star; 
    P_1_star = Lambda(P_1_star,theta,f_s,SS,AA,alpha_0); 
    if sum(abs(P(:)-P_1_star(:)))<convcons, break, end 
end 
if j >= 10000 
    disp('P does not converge under Lambda= after 1000 iterations. Stopping...'); 
    stop 
end 
P_0_star = 1 - P_1_star;
  
% (1)-(c) compute the ``optimal'' alpha to define Lambda mapping 
dP = dPsi_P(P_1_star,theta,f_s,SS,AA);
%dP = Psi_P('Psi',P_1_star,theta,f_s,SS,AA); 
eig_dp = eig(dP);   
options_fminsearch = optimset('Display','off','LargeScale','off','TolFun',sqrt(eps),'TolX',sqrt(eps),'MaxFunEvals',10000,'MaxIter',10000); 
b = fminsearch('eig_Lambda',1,options_fminsearch,eig_dp);   
options_fminsearch = optimset('Display','off','LargeScale','off','TolFun',sqrt(eps),'TolX',sqrt(eps),'MaxFunEvals',10000,'MaxIter',10000); 
b = fminunc('eig_Lambda',b,options_fminsearch,eig_dp);   
alpha = 1/(1+abs(b));  
disp('optimal value of alpha =');disp(alpha); 
  

% (2) Obtain the steady state distribution p^*(S_t,a_{t-1}) 
prod_P_star = zeros(ns,na); %Pr(a|x) = \prod_{j=1}^N P_j(a_j|x) 
for j = 1:na 
    prod_P_star(:,j) = ones(ns,1);
    for t = 1:nglob
        prod_P_star(:,j) =  prod_P_star(:,j).*(P_1_star(:,t).*AA(j,t)+P_0_star(:,t).*(1-AA(j,t)));
    end
end
F_P_star = kron(ones(na,na),f_s).*kron(prod_P_star,ones(1,nstate));
p_star = (1/ns)*ones(ns,1);
for j = 1:10000
    p_star_0 = p_star;
    p_star = F_P_star'*p_star_0;
    if sum(abs(p_star-p_star_0))<convcons, break, end
end
p_star = p_star./sum(p_star(:)); % to avoid numerical errors
  
% (3) Descriptive statistics for the model
p_entry = P_1_star'*p_star;
disp('Probability of being active [firm1, firm2, firm3]=');
disp(P_1_star'*p_star);
dP = dPsi_P(P_1_star,theta,f_s,SS,AA);
dP_Lambda = (1-alpha)*eye(ns*nglob,ns*nglob) + alpha*dP; 
%dP = Psi_P('Psi',P_1_star,theta,f_s,SS,AA); 
%dP_Lambda = Psi_P('Lambda',P_1_star,theta,f_s,SS,AA,alpha);  
dP_dtheta = dPsi_theta(P_1_star,theta,f_s,SS,AA);
dP_dtheta = [dP_dtheta;-dP_dtheta];
Delta_P = diag(repmat(p_star,2*nglob,1)./[P_1_star(:);1-P_1_star(:)]);
Proj = dP_dtheta*inv(dP_dtheta'*Delta_P*dP_dtheta)*dP_dtheta'*Delta_P;
M = eye(2*ns*nglob) - Proj; % M_Lambda = M
M_11 = M(1:ns*nglob,1:ns*nglob);
M_12 = M(1:ns*nglob,ns*nglob+1:2*ns*nglob);   
disp('Table 1: Dominant Eigenvalue of Psi and Lambda');  
disp('\theta_{RN}=');disp(theta_RN); 
disp('[alpha,\rho(Psi_P),\rho(Lambda_P),\rho(M*Psi_P),\rho(M*Lambda_P)]='); 
disp([alpha,max(abs(eig(dP))),max(abs(eig(dP_Lambda))),max(abs(eig((M_11-M_12)*dP))),max(abs(eig((M_11-M_12)*dP_Lambda)))]); 
disp('[alpha,||Psi_P||_s,||Lambda_P||_s,||M*Psi_P||_s,||M*Lambda_P||_s]='); 
disp([alpha,sqrt(max(abs(eig(dP'*dP)))),max(sqrt(abs(eig(dP_Lambda'*dP_Lambda)))),max(sqrt(abs(eig(((M_11-M_12)*dP)'*((M_11-M_12)*dP))))),max(sqrt(abs(eig(((M_11-M_12)*dP_Lambda)'*(M_11-M_12)*dP_Lambda))))]);

 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%  Solving the fixed points of Psi  %%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
% (4) Solving the fixed points of Psi
% (4.0) Draw initial values of P_0 from U[0.01,0.99]  
% (4.1) Solving the system of nonlinear equations: P-Psi(theta^0,P)=0  
%       using (i) c05nb.m from the NAG Toolbox for MATLAB. 
%             (ii) fsolve.m from Matlab Optimization Toolbox 
%                  using Trust-region dogleg algorithm 
% (4.2) Repeat (4.0)-(4.1) for 1000 simulated initial values for P_0
% Note: We provide the analytical derivatives for dPsi/dP in coding the
% objective function for fsolve.m

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%           Set ``options'' for optimization routines           %%%%%  
options_fsolve = optimset('Display','off','DerivativeCheck','off','Jacobian','on','Algorithm','trust-region-dogleg','TolFun',1e-8,'TolX',1e-8,'MaxFunEvals',1000,'MaxIter',1000); 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% (4.0) Initial values are uniformly distributed between 0.01 and 0.99 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%nsim = 1000;
P_sim = zeros(ns,nglob,nsim); 
for iter = 1:nsim 
    P_sim(:,:,iter) = 0.01*ones(ns,nglob)+0.98*rand(ns,nglob); % an initial P is uniformly distributed between 0.01 and 0.99 
end 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% (4.1) Solving the system of nonlinear equations: P-Psi(theta^0,P)=0  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% (i) using c05nb.m from the NAG Toolbox for MATLAB
P_iter_NAG = zeros(ns*nglob,nsim);
P_fval_NAG = zeros(ns*nglob,nsim);   
P_RMES_NAG = zeros(nsim,1);  
for iter = 1:nsim 
    P_0 = squeeze(P_sim(:,:,iter));
    [P_1,fval] = c05nb('Psi_fp_NAG',P_0(:)); 
    P_iter_NAG(:,iter) = P_1; 
    P_fval_NAG(:,iter) = fval; 
    P_RMES_NAG(iter,1) = sum((P_1-P_1_star(:)).^2);  
    %disp('[iter,SSR,||P_1(:)-P_1_star(:)||]='); 
    %disp([iter,sum(fval.^2),P_RMES_NAG(iter,1)]);  
   % report every 50 iterations to see the program is running 
   if iter/50 == floor(iter/50) 
       disp(iter); 
   end 
end 
sum_P_iter_NAG = 0;
sum_P_iter_NAG_diff = 0;
for iter = 1:nsim 
    if sum(P_fval_NAG(:,iter).^2)<1e-9
%         disp([iter,P_RMES_NAG(iter)]);
        sum_P_iter_NAG = sum_P_iter_NAG + 1;
        if P_RMES_NAG(iter)>1e-6
            sum_P_iter_NAG_diff = sum_P_iter_NAG_diff + 1;
        end
    end
end 
disp('a fraction of simulations that converged=');
disp(sum_P_iter_NAG/nsim);  
disp('a fraction of simulations that converged to a point different from P_1_star =');
disp(sum_P_iter_NAG_diff/nsim);  
 

%  (ii) fsolve.m from Matlab Optimization Toolbox using Trust-region dogleg algorithm 
P_iter = zeros(ns*nglob,nsim);
P_fval = zeros(ns*nglob,nsim);   
P_RMES = zeros(nsim,1);  
P_exit = zeros(nsim,1); 
for iter = 1:nsim 
    P_0 = squeeze(P_sim(:,:,iter));
    [P_1,fval,exitflag,output_fsolve] = fsolve('Psi_fp',P_0(:),options_fsolve,theta,f_s,SS,AA); 
    P_iter(:,iter) = P_1; 
    P_fval(:,iter) = fval; 
    P_RMES(iter,1) = sum((P_1-P_1_star(:)).^2); 
    P_exit(iter,1) = exitflag;
%     disp('[iter,SSR,||P_1(:)-P_1_star(:)||]='); 
%     disp([iter,sum(fval.^2),P_RMES(iter,1)]);  
   % report every 50 iterations to see the program is running 
   if iter/50 == floor(iter/50) 
       disp(iter); 
   end 
end 
sum_P_iter = 0;
sum_P_iter_diff = 0;
for iter = 1:nsim 
    if sum(P_fval(:,iter).^2)<1e-8
       % disp([sum(P_fval(:,iter).^2),P_exit(iter),P_RMES(iter)]);
        sum_P_iter = sum_P_iter + 1;
        if P_RMES(iter)>1e-6
            sum_P_iter_diff = sum_P_iter_diff + 1;
        end
    end
end 
disp('a fraction of simulations that converged=');
disp(sum_P_iter/nsim);  
disp('a fraction of simulations that converged to a point different from P_1_star =');
disp(sum_P_iter_diff/nsim);  
 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%  Solving the population NPL fixed points   %%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
% (5) Solving the population NPL fixed points  
% (5.0) Draw initial values of P_0 from U[0.01,0.09]  
% (5.1) Solving the system of nonlinear equations: phi_0(P)-P=0 
%       using (i) c05nb.m from the NAG Toolbox for MATLAB. 
%             (ii) fsolve.m from Matlab Optimization Toolbox 
%                  using Trust-region dogleg algorithm  
% (5.2) Repeat (5.0)-(5.1) for 1000 simulated initial values for P_0
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%   Set ``options'' for optimization routines       %%%%%  
options_fsolve = optimset('Display','off','Jacobian','off','Algorithm','trust-region-dogleg','TolFun',1e-8,'TolX',1e-8,'MaxFunEvals',1000,'MaxIter',1000);   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 

% (5.1.1) Solving a fixed point by solving the system of 
% nonlinear equations, phi_0(P)-P=0, using c05nb.m  
P_phi_iter_NAG = zeros(ns*nglob,nsim);
P_phi_fval_NAG = zeros(ns*nglob,nsim);   
P_phi_RMES_NAG = zeros(nsim,1);  
for iter = 1:nsim 
    P_0 = squeeze(P_sim(:,:,iter));
    b_j = zeros(2,4); % try four different initial values of b's 
    for i = 1:4 
        b_j(:,i) = theta(1:2,1) + (2-4*rand(2,1)); 
    end   
    [P_1,fval] = c05nb('f_phi_population_NAG',P_0(:)); 
    P_phi_iter_NAG(:,iter) = P_1; 
    P_phi_fval_NAG(:,iter) = fval; 
    P_phi_RMES_NAG(iter,1) = sum((P_1-P_1_star(:)).^2);  
%     disp('[iter,SSR,||P_1(:)-P_1_star(:)||]='); 
%     disp([iter,sum(fval.^2),P_phi_RMES_NAG(iter,1)]);   
   if iter/50 == floor(iter/50) 
       disp(iter); 
   end 
end   
sum_P_phi_iter_NAG = 0;
sum_P_phi_iter_NAG_diff = 0;
for iter = 1:nsim 
    if sum(P_phi_fval_NAG(:,iter).^2)<1e-9
%         disp([iter,P_phi_RMES_NAG(iter)]);
        sum_P_phi_iter_NAG = sum_P_phi_iter_NAG + 1;
        if P_phi_RMES_NAG(iter)>1e-6
            sum_P_phi_iter_NAG_diff = sum_P_phi_iter_NAG_diff + 1;
        end
    end
end 
disp('a fraction of simulations that converged=');
disp(sum_P_phi_iter_NAG/nsim);  
disp('a fraction of simulations that converged to a point different from P_1_star =');
disp(sum_P_phi_iter_NAG_diff/nsim);   

% (5.1.2) Solving a fixed point by solving the system of 
% nonlinear equations, phi_0(P)-P=0, using fsolve.m   
P_phi_iter = zeros(ns*nglob,nsim);
P_phi_fval = zeros(ns*nglob,nsim);   
P_phi_RMES = zeros(nsim,1);  
P_phi_exit = zeros(nsim,1); 
for iter = 1:nsim 
    P_0 = squeeze(P_sim(:,:,iter)); 
    b_j = zeros(2,4); % try four different initial values of b's 
    for i = 1:4 
        b_j(:,i) = theta(1:2,1) + (2-4*rand(2,1)); 
    end   
    [P_1,fval,exitflag,output_fsolve] = fsolve('f_phi_population',P_0(:),options_fsolve,theta,b_j,p_star,P_1_star,f_s,SS,AA);  
    P_phi_iter(:,iter) = P_1; 
    P_phi_fval(:,iter) = fval; 
    P_phi_RMES(iter,1) = sum((P_1-P_1_star(:)).^2); 
    P_phi_exit(iter,1) = exitflag;
%     disp('[iter,SSR,||P_1(:)-P_1_star(:)||]='); 
%     disp([iter,sum(fval.^2),P_RMES(iter,1)]);   
   % report every 50 iterations to see the program is running 
   if iter/50 == floor(iter/50) 
       disp(iter); 
   end  
end 
sum_P_phi_iter = 0;
sum_P_phi_iter_diff = 0;
for iter = 1:nsim 
    if sum(P_phi_fval(:,iter).^2)<1e-8
       % disp([sum(P_phi_fval(:,iter).^2),P_phi_exit(iter),P_phi_RMES(iter)]);
        sum_P_phi_iter = sum_P_phi_iter + 1;
        if P_phi_RMES(iter)>1e-6
            sum_P_phi_iter_diff = sum_P_phi_iter_diff + 1;
        end
    end
end 
disp('a fraction of simulations that converged=');
disp(sum_P_phi_iter/nsim);  
disp('a fraction of simulations that converged to a point different from P_1_star =');
disp(sum_P_phi_iter_diff/nsim);  
   

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%   Reporting the results of simulation    %%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
diary('mpe_population_result.txt'); 
diary on  
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%'); 
disp('%%%%%%%%%%%%%        Simulation Results              %%%%%%%%%%%%%'); 
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%'); 
disp('  '); 
disp('Simulation Setup '); 
disp('  '); 
disp('theta_RN='); 
disp(theta_RN); 
disp('  ');  
disp('The number of trial initial values of P_0'); 
disp(nsim);  
disp('  ');  
  
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%'); 
disp('%%%   Simulation Results for Finding the Fixed Points of Psi   %%%'); 
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%'); 
 
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
disp(' Using c05nb.m from the NAG Toolbox for MATLAB  ');
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%'); 
disp('a fraction of simulations in which c05nb.m finds a fixed point of P=Psi(theta^0,P)');
disp(sum_P_iter_NAG/nsim);  
disp('a fraction of simulations in which c05nb.m finds a fixed point of P=Psi(theta^0,P) that is different from P_1_star');
disp(sum_P_iter_NAG_diff/nsim);    
 
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
disp(' Using fsolve.m from the MATLAB Optimization Toolbox  ');
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%'); 
disp('a fraction of simulations in which fsolve.m finds a fixed point of P=Psi(theta^0,P)');
disp(sum_P_iter/nsim);  
disp('a fraction of simulations in which fsolve.m finds a fixed point of P=Psi(theta^0,P) that is different from P_1_star');
disp(sum_P_iter_diff/nsim);   


disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%'); 
disp('%%%   Simulation Results for Finding the Fixed Points of phi_0(P)-P=0   %%%'); 
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%'); 
 
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
disp(' Using c05nb.m from the NAG Toolbox for MATLAB  ');
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%'); 
disp('a fraction of simulations in which c05nb.m finds a fixed point of phi_0(P)=P');
disp(sum_P_phi_iter_NAG/nsim); 
disp('a fraction of simulations in which c05nb.m finds a fixed point of phi_0(P)=P that is different from P_1_star');
disp(sum_P_phi_iter_NAG_diff/nsim);   
 
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%'); 
disp(' Using fsolve.m from the MATLAB Optimization Toolbox  '); 
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%'); 
disp('a fraction of simulations in which fsolve.m finds a fixed point of phi_0(P)=P');
disp(sum_P_phi_iter/nsim);  
disp('a fraction of simulations in which fsolve.m finds a fixed point of phi_0(P)=P that is different from P_1_star');
disp(sum_P_phi_iter_diff/nsim);    

diary off



return